@@ -8,6 +8,7 @@
 #define mozilla_PresShellForwards_h
 
 #include "mozilla/TypedEnumBits.h"
+#include "mozilla/Maybe.h"
 
 struct CapturingContentInfo;
 
@@ -36,21 +37,18 @@ enum class ResizeReflowOptions : uint32_t {
   // the resulting BSize can be less than the given one, producing
   // shrink-to-fit sizing in the block dimension
   BSizeLimit = 1 << 0,
-  // Invalidate layout, but don't reflow.
-  //
-  // TODO(emilio): Ideally this should just become the default, or we should
-  // unconditionally not reflow and rely on the caller to do so, having a
-  // separate API for shrink-to-fit.
-  SuppressReflow = 1 << 1,
 };
 
 MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(ResizeReflowOptions)
 
 enum class IntrinsicDirty {
-  // XXXldb eResize should be renamed
-  Resize,       // don't mark any intrinsic widths dirty
-  TreeChange,   // mark intrinsic widths dirty on aFrame and its ancestors
-  StyleChange,  // Do eTreeChange, plus all of aFrame's descendants
+  // Don't mark any intrinsic inline sizes dirty.
+  None,
+  // Mark intrinsic inline sizes dirty on aFrame and its ancestors.
+  FrameAndAncestors,
+  // Mark intrinsic inline sizes dirty on aFrame, its ancestors, and its
+  // descendants.
+  FrameAncestorsAndDescendants,
 };
 
 enum class ReflowRootHandling {
@@ -58,20 +56,31 @@ enum class ReflowRootHandling {
   NoPositionOrSizeChange,  // ... NOT changing ...
   InferFromBitToAdd,       // is changing iff (aBitToAdd == NS_FRAME_IS_DIRTY)
 
-  // Note:  With eStyleChange, these can also apply to out-of-flows
-  // in addition to aFrame.
+  // Note:  With IntrinsicDirty::FrameAncestorsAndDescendants, these can also
+  // apply to out-of-flows in addition to aFrame.
 };
 
-// WhereToScroll should be 0 ~ 100 or -1.  When it's in 0 ~ 100, it means
-// percentage of scrollTop/scrollLeft in scrollHeight/scrollWidth.
-// See the comment for constructor of ScrollAxis for the detail.
-typedef int16_t WhereToScroll;
-static const WhereToScroll kScrollToTop = 0;
-static const WhereToScroll kScrollToLeft = 0;
-static const WhereToScroll kScrollToCenter = 50;
-static const WhereToScroll kScrollToBottom = 100;
-static const WhereToScroll kScrollToRight = 100;
-static const WhereToScroll kScrollMinimum = -1;
+// Indicates where to scroll on a given axis.
+struct WhereToScroll {
+  // The percentage of the scroll axis that we're scrolling to.
+  // Nothing() represents "scroll to nearest".
+  Maybe<int16_t> mPercentage;
+
+  // Default is nearest.
+  constexpr WhereToScroll() = default;
+
+  explicit constexpr WhereToScroll(int16_t aPercentage)
+      : mPercentage(Some(aPercentage)) {}
+
+  enum { Nearest };
+  MOZ_IMPLICIT constexpr WhereToScroll(decltype(Nearest)) : WhereToScroll() {}
+  enum { Start };
+  MOZ_IMPLICIT constexpr WhereToScroll(decltype(Start)) : WhereToScroll(0) {}
+  enum { Center };
+  MOZ_IMPLICIT constexpr WhereToScroll(decltype(Center)) : WhereToScroll(50) {}
+  enum { End };
+  MOZ_IMPLICIT constexpr WhereToScroll(decltype(End)) : WhereToScroll(100) {}
+};
 
 // See the comment for constructor of ScrollAxis for the detail.
 enum class WhenToScroll : uint8_t {
@@ -116,7 +125,7 @@ struct ScrollAxis final {
    *   scrollbar showing and less than one device pixel of scrollable
    *   distance), don't scroll. Defaults to false.
    */
-  explicit ScrollAxis(WhereToScroll aWhere = kScrollMinimum,
+  explicit ScrollAxis(WhereToScroll aWhere = WhereToScroll::Nearest,
                       WhenToScroll aWhen = WhenToScroll::IfNotFullyVisible,
                       bool aOnlyIfPerceivedScrollableDirection = false)
       : mWhereToScroll(aWhere),
@@ -129,16 +138,19 @@ struct ScrollAxis final {
   bool mOnlyIfPerceivedScrollableDirection : 1;
 };
 
-enum class ScrollFlags {
+enum class ScrollFlags : uint8_t {
   None = 0,
   ScrollFirstAncestorOnly = 1 << 0,
   ScrollOverflowHidden = 1 << 1,
   ScrollNoParentFrames = 1 << 2,
   ScrollSmooth = 1 << 3,
   ScrollSmoothAuto = 1 << 4,
-  ScrollSnap = 1 << 5,
-  // ScrollOverflowHidden | ScrollNoParentFrames
-  AnchorScrollFlags = (1 << 1) | (1 << 2),
+  TriggeredByScript = 1 << 5,
+  AxesAreLogical = 1 << 6,
+  // NOTE: `Anchor` means here is "scrolling to an anchor", not "CSS scroll
+  // anchoring".
+  AnchorScrollFlags =
+      ScrollOverflowHidden | ScrollNoParentFrames | TriggeredByScript,
   ALL_BITS = (1 << 7) - 1,
 };
 
@@ -176,32 +188,23 @@ enum class ResolutionChangeOrigin : uint8_t {
   MainThreadAdjustment,
 };
 
-// See comment at declaration of AddCanvasBackgroundColorItem() for the detail.
-enum class AddCanvasBackgroundColorFlags {
-  None = 0,
-  ForceDraw = 1 << 0,
-  AddForSubDocument = 1 << 1,
-  AppendUnscrolledOnly = 1 << 2,
-};
-
-MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(AddCanvasBackgroundColorFlags)
-
 enum class PaintFlags {
   None = 0,
-  /* Update the layer tree and paint PaintedLayers. If this is not specified,
-   * we may still have to do it if the layer tree lost PaintedLayer contents
-   * we need for compositing. */
-  PaintLayers = 1 << 0,
-  /* Composite layers to the window. */
-  PaintComposite = 1 << 1,
   /* Sync-decode images. */
-  PaintSyncDecodeImages = 1 << 2,
+  PaintSyncDecodeImages = 1 << 1,
 };
 
 MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(PaintFlags)
 
-// See comment at declaration of ScheduleViewManagerFlush() for the detail.
-enum class PaintType { Default, DelayedCompress };
+enum class PaintInternalFlags {
+  None = 0,
+  /* Sync-decode images. */
+  PaintSyncDecodeImages = 1 << 1,
+  /* Composite layers to the window. */
+  PaintComposite = 1 << 2,
+};
+
+MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(PaintInternalFlags)
 
 // This is a private enum class of PresShell, but currently,
 // MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS isn't available in class definition.
