@@ -7,10 +7,19 @@
 
 #include "TouchManager.h"
 
-#include "gfxPrefs.h"
-#include "mozilla/dom/EventTarget.h"
+#include "Units.h"
+#include "mozilla/EventForwards.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/StaticPrefs_apz.h"
+#include "mozilla/StaticPrefs_test.h"
+#include "mozilla/TimeStamp.h"
+#include "mozilla/dom/Document.h"
+#include "mozilla/dom/EventTarget.h"
+#include "mozilla/dom/PointerEventHandler.h"
+#include "mozilla/layers/InputAPZContext.h"
+#include "nsIContent.h"
 #include "nsIFrame.h"
+#include "nsLayoutUtils.h"
 #include "nsView.h"
 #include "PositionedEventTargeting.h"
 
@@ -18,55 +27,61 @@ using namespace mozilla::dom;
 
 namespace mozilla {
 
-nsDataHashtable<nsUint32HashKey, TouchManager::TouchInfo>*
+StaticAutoPtr<nsTHashMap<nsUint32HashKey, TouchManager::TouchInfo>>
     TouchManager::sCaptureTouchList;
+layers::LayersId TouchManager::sCaptureTouchLayersId;
+TimeStamp TouchManager::sSingleTouchStartTimeStamp;
+LayoutDeviceIntPoint TouchManager::sSingleTouchStartPoint;
+bool TouchManager::sPrecedingTouchPointerDownConsumedByContent = false;
 
-/*static*/ void TouchManager::InitializeStatics() {
+/*static*/
+void TouchManager::InitializeStatics() {
   NS_ASSERTION(!sCaptureTouchList, "InitializeStatics called multiple times!");
-  sCaptureTouchList =
-      new nsDataHashtable<nsUint32HashKey, TouchManager::TouchInfo>;
+  sCaptureTouchList = new nsTHashMap<nsUint32HashKey, TouchManager::TouchInfo>;
+  sCaptureTouchLayersId = layers::LayersId{0};
 }
 
-/*static*/ void TouchManager::ReleaseStatics() {
+/*static*/
+void TouchManager::ReleaseStatics() {
   NS_ASSERTION(sCaptureTouchList, "ReleaseStatics called without Initialize!");
-  delete sCaptureTouchList;
   sCaptureTouchList = nullptr;
 }
 
-void TouchManager::Init(PresShell* aPresShell, nsIDocument* aDocument) {
+void TouchManager::Init(PresShell* aPresShell, Document* aDocument) {
   mPresShell = aPresShell;
   mDocument = aDocument;
 }
 
 void TouchManager::Destroy() {
-  EvictTouches();
+  EvictTouches(mDocument);
   mDocument = nullptr;
   mPresShell = nullptr;
 }
 
 static nsIContent* GetNonAnonymousAncestor(EventTarget* aTarget) {
-  nsCOMPtr<nsIContent> content(do_QueryInterface(aTarget));
+  nsIContent* content = nsIContent::FromEventTargetOrNull(aTarget);
   if (content && content->IsInNativeAnonymousSubtree()) {
     content = content->FindFirstNonChromeOnlyAccessContent();
   }
   return content;
 }
 
-/*static*/ void TouchManager::EvictTouchPoint(RefPtr<Touch>& aTouch,
-                                              nsIDocument* aLimitToDocument) {
-  nsCOMPtr<nsINode> node(do_QueryInterface(aTouch->mTarget));
+/*static*/
+void TouchManager::EvictTouchPoint(RefPtr<Touch>& aTouch,
+                                   Document* aLimitToDocument) {
+  nsCOMPtr<nsINode> node(
+      nsINode::FromEventTargetOrNull(aTouch->mOriginalTarget));
   if (node) {
-    nsIDocument* doc = node->GetUncomposedDoc();
+    Document* doc = node->GetComposedDoc();
     if (doc && (!aLimitToDocument || aLimitToDocument == doc)) {
-      nsIPresShell* presShell = doc->GetShell();
+      PresShell* presShell = doc->GetPresShell();
       if (presShell) {
         nsIFrame* frame = presShell->GetRootFrame();
         if (frame) {
-          nsPoint pt(aTouch->mRefPoint.x, aTouch->mRefPoint.y);
-          nsCOMPtr<nsIWidget> widget = frame->GetView()->GetNearestWidget(&pt);
+          nsCOMPtr<nsIWidget> widget =
+              frame->GetView()->GetNearestWidget(nullptr);
           if (widget) {
             WidgetTouchEvent event(true, eTouchEnd, widget);
-            event.mTime = PR_IntervalNow();
             event.mTouches.AppendElement(aTouch);
             nsEventStatus status;
             widget->DispatchEvent(&event, status);
@@ -80,25 +95,28 @@ static nsIContent* GetNonAnonymousAncestor(EventTarget* aTarget) {
   }
 }
 
-/*static*/ void TouchManager::AppendToTouchList(
-    WidgetTouchEvent::TouchArray* aTouchList) {
-  for (auto iter = sCaptureTouchList->Iter(); !iter.Done(); iter.Next()) {
-    RefPtr<Touch>& touch = iter.Data().mTouch;
+/*static*/
+void TouchManager::AppendToTouchList(
+    WidgetTouchEvent::TouchArrayBase* aTouchList) {
+  for (const auto& data : sCaptureTouchList->Values()) {
+    const RefPtr<Touch>& touch = data.mTouch;
     touch->mChanged = false;
     aTouchList->AppendElement(touch);
   }
 }
 
-void TouchManager::EvictTouches() {
+void TouchManager::EvictTouches(Document* aLimitToDocument) {
   WidgetTouchEvent::AutoTouchArray touches;
   AppendToTouchList(&touches);
   for (uint32_t i = 0; i < touches.Length(); ++i) {
-    EvictTouchPoint(touches[i], mDocument);
+    EvictTouchPoint(touches[i], aLimitToDocument);
   }
+  sCaptureTouchLayersId = layers::LayersId{0};
 }
 
-/* static */ nsIFrame* TouchManager::SetupTarget(WidgetTouchEvent* aEvent,
-                                                 nsIFrame* aFrame) {
+/* static */
+nsIFrame* TouchManager::SetupTarget(WidgetTouchEvent* aEvent,
+                                    nsIFrame* aFrame) {
   MOZ_ASSERT(aEvent);
 
   if (!aEvent || aEvent->mMessage != eTouchStart) {
@@ -106,14 +124,6 @@ void TouchManager::EvictTouches() {
     return aFrame;
   }
 
-  uint32_t flags = 0;
-  // Setting this flag will skip the scrollbars on the root frame from
-  // participating in hit-testing, and we only want that to happen on
-  // zoomable platforms (for now).
-  if (gfxPrefs::APZAllowZooming()) {
-    flags |= INPUT_IGNORE_ROOT_SCROLL_FRAME;
-  }
-
   nsIFrame* target = aFrame;
   for (int32_t i = aEvent->mTouches.Length(); i;) {
     --i;
@@ -122,16 +132,16 @@ void TouchManager::EvictTouches() {
     int32_t id = touch->Identifier();
     if (!TouchManager::HasCapturedTouch(id)) {
       // find the target for this touch
+      RelativeTo relativeTo{aFrame};
       nsPoint eventPoint = nsLayoutUtils::GetEventCoordinatesRelativeTo(
-          aEvent, touch->mRefPoint, aFrame);
-      target = FindFrameTargetedByInputEvent(aEvent, aFrame, eventPoint, flags);
+          aEvent, touch->mRefPoint, relativeTo);
+      target = FindFrameTargetedByInputEvent(aEvent, relativeTo, eventPoint);
       if (target) {
         nsCOMPtr<nsIContent> targetContent;
         target->GetContentForEvent(aEvent, getter_AddRefs(targetContent));
-        while (targetContent && !targetContent->IsElement()) {
-          targetContent = targetContent->GetParent();
-        }
-        touch->SetTarget(targetContent);
+        touch->SetTouchTarget(targetContent
+                                  ? targetContent->GetAsElementOrParentElement()
+                                  : nullptr);
       } else {
         aEvent->mTouches.RemoveElementAt(i);
       }
@@ -141,14 +151,15 @@ void TouchManager::EvictTouches() {
       touch->mChanged = false;
       RefPtr<dom::Touch> oldTouch = TouchManager::GetCapturedTouch(id);
       if (oldTouch) {
-        touch->SetTarget(oldTouch->mTarget);
+        touch->SetTouchTarget(oldTouch->mOriginalTarget);
       }
     }
   }
   return target;
 }
 
-/* static */ nsIFrame* TouchManager::SuppressInvalidPointsAndGetTargetedFrame(
+/* static */
+nsIFrame* TouchManager::SuppressInvalidPointsAndGetTargetedFrame(
     WidgetTouchEvent* aEvent) {
   MOZ_ASSERT(aEvent);
 
@@ -166,15 +177,42 @@ void TouchManager::EvictTouches() {
   }
 
   nsIFrame* frame = nullptr;
-  for (int32_t i = aEvent->mTouches.Length(); i;) {
+  for (uint32_t i = aEvent->mTouches.Length(); i;) {
     --i;
     dom::Touch* touch = aEvent->mTouches[i];
     if (TouchManager::HasCapturedTouch(touch->Identifier())) {
       continue;
     }
 
-    MOZ_ASSERT(touch->mTarget);
-    nsCOMPtr<nsIContent> targetContent = do_QueryInterface(touch->GetTarget());
+    MOZ_ASSERT(touch->mOriginalTarget);
+    nsIContent* const targetContent =
+        nsIContent::FromEventTargetOrNull(touch->GetTarget());
+    if (MOZ_UNLIKELY(!targetContent)) {
+      touch->mIsTouchEventSuppressed = true;
+      continue;
+    }
+
+    // Even if the target content is not connected, we should dispatch the touch
+    // start event except when the target content is owned by different
+    // document.
+    if (MOZ_UNLIKELY(!targetContent->IsInComposedDoc())) {
+      if (anyTarget && anyTarget->OwnerDoc() != targetContent->OwnerDoc()) {
+        touch->mIsTouchEventSuppressed = true;
+        continue;
+      }
+      if (!anyTarget) {
+        anyTarget = targetContent;
+      }
+      touch->SetTouchTarget(targetContent->GetAsElementOrParentElement());
+      if (PresShell* const presShell =
+              targetContent->OwnerDoc()->GetPresShell()) {
+        if (nsIFrame* rootFrame = presShell->GetRootFrame()) {
+          frame = rootFrame;
+        }
+      }
+      continue;
+    }
+
     nsIFrame* targetFrame = targetContent->GetPrimaryFrame();
     if (targetFrame && !anyTarget) {
       anyTarget = targetContent;
@@ -199,11 +237,12 @@ void TouchManager::EvictTouches() {
         touch->mIsTouchEventSuppressed = true;
       } else {
         targetFrame = newTargetFrame;
-        targetFrame->GetContentForEvent(aEvent, getter_AddRefs(targetContent));
-        while (targetContent && !targetContent->IsElement()) {
-          targetContent = targetContent->GetParent();
-        }
-        touch->SetTarget(targetContent);
+        nsCOMPtr<nsIContent> newTargetContent;
+        targetFrame->GetContentForEvent(aEvent,
+                                        getter_AddRefs(newTargetContent));
+        touch->SetTouchTarget(
+            newTargetContent ? newTargetContent->GetAsElementOrParentElement()
+                             : nullptr);
       }
     }
     if (targetFrame) {
@@ -214,21 +253,35 @@ void TouchManager::EvictTouches() {
 }
 
 bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
-                                  bool& aTouchIsNew, bool& aIsHandlingUserInput,
+                                  bool& aTouchIsNew,
                                   nsCOMPtr<nsIContent>& aCurrentEventContent) {
+  MOZ_DIAGNOSTIC_ASSERT(aEvent->IsTrusted());
+
+  // NOTE: If you need to handle new event messages here, you need to add new
+  //       cases in PresShell::EventHandler::PrepareToDispatchEvent().
   switch (aEvent->mMessage) {
     case eTouchStart: {
-      aIsHandlingUserInput = true;
       WidgetTouchEvent* touchEvent = aEvent->AsTouchEvent();
       // if there is only one touch in this touchstart event, assume that it is
       // the start of a new touch session and evict any old touches in the
       // queue
       if (touchEvent->mTouches.Length() == 1) {
-        WidgetTouchEvent::AutoTouchArray touches;
-        AppendToTouchList(&touches);
-        for (uint32_t i = 0; i < touches.Length(); ++i) {
-          EvictTouchPoint(touches[i]);
-        }
+        EvictTouches();
+        // Per
+        // https://w3c.github.io/touch-events/#touchevent-implementer-s-note,
+        // all touch event should be dispatched to the same document that first
+        // touch event associated to. We cache layers id of the first touchstart
+        // event, all subsequent touch events will use the same layers id.
+        sCaptureTouchLayersId = aEvent->mLayersId;
+        sSingleTouchStartTimeStamp = aEvent->mTimeStamp;
+        sSingleTouchStartPoint = touchEvent->mTouches[0]->mRefPoint;
+        const PointerInfo* pointerInfo = PointerEventHandler::GetPointerInfo(
+            touchEvent->mTouches[0]->Identifier());
+        sPrecedingTouchPointerDownConsumedByContent =
+            pointerInfo && pointerInfo->mPreventMouseEventByContent;
+      } else {
+        touchEvent->mLayersId = sCaptureTouchLayersId;
+        sSingleTouchStartTimeStamp = TimeStamp();
       }
       // Add any new touches to the queue
       WidgetTouchEvent::TouchArray& touches = touchEvent->mTouches;
@@ -241,8 +294,9 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
           touch->mChanged = true;
         }
         touch->mMessage = aEvent->mMessage;
-        TouchInfo info = {touch, GetNonAnonymousAncestor(touch->mTarget), true};
-        sCaptureTouchList->Put(id, info);
+        TouchInfo info = {
+            touch, GetNonAnonymousAncestor(touch->mOriginalTarget), true};
+        sCaptureTouchList->InsertOrUpdate(id, info);
         if (touch->mIsTouchEventSuppressed) {
           // We're going to dispatch touch event. Remove this touch instance if
           // it is suppressed.
@@ -256,6 +310,7 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
       // Check for touches that changed. Mark them add to queue
       WidgetTouchEvent* touchEvent = aEvent->AsTouchEvent();
       WidgetTouchEvent::TouchArray& touches = touchEvent->mTouches;
+      touchEvent->mLayersId = sCaptureTouchLayersId;
       bool haveChanged = false;
       for (int32_t i = touches.Length(); i;) {
         --i;
@@ -271,26 +326,26 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
           touches.RemoveElementAt(i);
           continue;
         }
-        RefPtr<Touch> oldTouch = info.mTouch;
-        if (!touch->Equals(oldTouch)) {
+        const RefPtr<Touch> oldTouch = info.mTouch;
+        if (!oldTouch->Equals(touch)) {
           touch->mChanged = true;
           haveChanged = true;
         }
 
-        nsCOMPtr<EventTarget> targetPtr = oldTouch->mTarget;
+        nsCOMPtr<EventTarget> targetPtr = oldTouch->mOriginalTarget;
         if (!targetPtr) {
           touches.RemoveElementAt(i);
           continue;
         }
         nsCOMPtr<nsINode> targetNode(do_QueryInterface(targetPtr));
         if (!targetNode->IsInComposedDoc()) {
-          targetPtr = do_QueryInterface(info.mNonAnonymousTarget);
+          targetPtr = info.mNonAnonymousTarget;
         }
-        touch->SetTarget(targetPtr);
+        touch->SetTouchTarget(targetPtr);
 
         info.mTouch = touch;
         // info.mNonAnonymousTarget is still valid from above
-        sCaptureTouchList->Put(id, info);
+        sCaptureTouchList->InsertOrUpdate(id, info);
         // if we're moving from touchstart to touchmove for this touch
         // we allow preventDefault to prevent mouse events
         if (oldTouch->mMessage != touch->mMessage) {
@@ -318,20 +373,22 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
             }
           }
         } else {
+          // This touch event isn't going to be dispatched on the main-thread,
+          // we need to tell it to APZ because returned nsEventStatus is
+          // unreliable to tell whether the event was preventDefaulted or not.
+          layers::InputAPZContext::SetDropped();
           return false;
         }
       }
       break;
     }
     case eTouchEnd:
-      aIsHandlingUserInput = true;
-      // Fall through to touchcancel code
-      MOZ_FALLTHROUGH;
     case eTouchCancel: {
       // Remove the changed touches
       // need to make sure we only remove touches that are ending here
       WidgetTouchEvent* touchEvent = aEvent->AsTouchEvent();
       WidgetTouchEvent::TouchArray& touches = touchEvent->mTouches;
+      touchEvent->mLayersId = sCaptureTouchLayersId;
       for (int32_t i = touches.Length(); i;) {
         --i;
         Touch* touch = touches[i];
@@ -346,14 +403,14 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
         if (!sCaptureTouchList->Get(id, &info)) {
           continue;
         }
-        nsCOMPtr<EventTarget> targetPtr = info.mTouch->mTarget;
+        nsCOMPtr<EventTarget> targetPtr = info.mTouch->mOriginalTarget;
         nsCOMPtr<nsINode> targetNode(do_QueryInterface(targetPtr));
         if (targetNode && !targetNode->IsInComposedDoc()) {
-          targetPtr = do_QueryInterface(info.mNonAnonymousTarget);
+          targetPtr = info.mNonAnonymousTarget;
         }
 
         aCurrentEventContent = do_QueryInterface(targetPtr);
-        touch->SetTarget(targetPtr);
+        touch->SetTouchTarget(targetPtr);
         sCaptureTouchList->Remove(id);
         if (info.mTouch->mIsTouchEventSuppressed) {
           touches.RemoveElementAt(i);
@@ -369,6 +426,7 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
       // is received.
       WidgetTouchEvent* touchEvent = aEvent->AsTouchEvent();
       WidgetTouchEvent::TouchArray& touches = touchEvent->mTouches;
+      touchEvent->mLayersId = sCaptureTouchLayersId;
       for (uint32_t i = 0; i < touches.Length(); ++i) {
         Touch* touch = touches[i];
         if (!touch) {
@@ -380,7 +438,7 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
           continue;
         }
         info.mConvertToPointer = false;
-        sCaptureTouchList->Put(id, info);
+        sCaptureTouchList->InsertOrUpdate(id, info);
       }
       break;
     }
@@ -390,18 +448,72 @@ bool TouchManager::PreHandleEvent(WidgetEvent* aEvent, nsEventStatus* aStatus,
   return true;
 }
 
-/*static*/ already_AddRefed<nsIContent>
-TouchManager::GetAnyCapturedTouchTarget() {
+void TouchManager::PostHandleEvent(const WidgetEvent* aEvent,
+                                   const nsEventStatus* aStatus) {
+  switch (aEvent->mMessage) {
+    case eTouchMove: {
+      if (sSingleTouchStartTimeStamp.IsNull()) {
+        break;
+      }
+      if (*aStatus == nsEventStatus_eConsumeNoDefault) {
+        sSingleTouchStartTimeStamp = TimeStamp();
+        break;
+      }
+      const WidgetTouchEvent* touchEvent = aEvent->AsTouchEvent();
+      if (touchEvent->mTouches.Length() > 1) {
+        sSingleTouchStartTimeStamp = TimeStamp();
+        break;
+      }
+      if (touchEvent->mTouches.Length() == 1) {
+        // If the touch moved too far from the start point, don't treat the
+        // touch as a tap.
+        const float distance =
+            static_cast<float>((sSingleTouchStartPoint -
+                                aEvent->AsTouchEvent()->mTouches[0]->mRefPoint)
+                                   .Length());
+        const float maxDistance =
+            StaticPrefs::apz_touch_start_tolerance() *
+            (MOZ_LIKELY(touchEvent->mWidget) ? touchEvent->mWidget->GetDPI()
+                                             : 96.0f);
+        if (distance > maxDistance) {
+          sSingleTouchStartTimeStamp = TimeStamp();
+        }
+      }
+      break;
+    }
+    case eTouchStart:
+    case eTouchEnd:
+      if (*aStatus == nsEventStatus_eConsumeNoDefault &&
+          !sSingleTouchStartTimeStamp.IsNull()) {
+        sSingleTouchStartTimeStamp = TimeStamp();
+      }
+      break;
+    case eTouchCancel:
+    case eTouchPointerCancel:
+    case eMouseLongTap:
+    case eContextMenu: {
+      if (!sSingleTouchStartTimeStamp.IsNull()) {
+        sSingleTouchStartTimeStamp = TimeStamp();
+      }
+      break;
+    }
+    default:
+      break;
+  }
+}
+
+/*static*/
+already_AddRefed<nsIContent> TouchManager::GetAnyCapturedTouchTarget() {
   nsCOMPtr<nsIContent> result = nullptr;
   if (sCaptureTouchList->Count() == 0) {
     return result.forget();
   }
-  for (auto iter = sCaptureTouchList->Iter(); !iter.Done(); iter.Next()) {
-    RefPtr<Touch>& touch = iter.Data().mTouch;
+  for (const auto& data : sCaptureTouchList->Values()) {
+    const RefPtr<Touch>& touch = data.mTouch;
     if (touch) {
       EventTarget* target = touch->GetTarget();
       if (target) {
-        result = do_QueryInterface(target);
+        result = nsIContent::FromEventTargetOrNull(target);
         break;
       }
     }
@@ -409,11 +521,13 @@ TouchManager::GetAnyCapturedTouchTarget() {
   return result.forget();
 }
 
-/*static*/ bool TouchManager::HasCapturedTouch(int32_t aId) {
+/*static*/
+bool TouchManager::HasCapturedTouch(int32_t aId) {
   return sCaptureTouchList->Contains(aId);
 }
 
-/*static*/ already_AddRefed<Touch> TouchManager::GetCapturedTouch(int32_t aId) {
+/*static*/
+already_AddRefed<Touch> TouchManager::GetCapturedTouch(int32_t aId) {
   RefPtr<Touch> touch;
   TouchInfo info;
   if (sCaptureTouchList->Get(aId, &info)) {
@@ -422,8 +536,9 @@ TouchManager::GetAnyCapturedTouchTarget() {
   return touch.forget();
 }
 
-/*static*/ bool TouchManager::ShouldConvertTouchToPointer(
-    const Touch* aTouch, const WidgetTouchEvent* aEvent) {
+/*static*/
+bool TouchManager::ShouldConvertTouchToPointer(const Touch* aTouch,
+                                               const WidgetTouchEvent* aEvent) {
   if (!aTouch || !aTouch->convertToPointer) {
     return false;
   }
@@ -437,7 +552,49 @@ TouchManager::GetAnyCapturedTouchTarget() {
     // pre-handling touch events.
     return aEvent->mMessage == eTouchStart;
   }
-  return info.mConvertToPointer;
+
+  if (!info.mConvertToPointer) {
+    return false;
+  }
+
+  switch (aEvent->mMessage) {
+    case eTouchStart: {
+      // We don't want to fire duplicated pointerdown.
+      return false;
+    }
+    case eTouchMove: {
+      return !aTouch->Equals(info.mTouch);
+    }
+    default:
+      break;
+  }
+  return true;
+}
+
+/* static */
+bool TouchManager::IsSingleTapEndToDoDefault(
+    const WidgetTouchEvent* aTouchEndEvent) {
+  MOZ_ASSERT(aTouchEndEvent);
+  MOZ_ASSERT(aTouchEndEvent->mFlags.mIsSynthesizedForTests);
+  MOZ_ASSERT(!StaticPrefs::test_events_async_enabled());
+  if (sSingleTouchStartTimeStamp.IsNull() ||
+      aTouchEndEvent->mTouches.Length() != 1) {
+    return false;
+  }
+  // If it's pressed long time, we should not treat it as a single tap because
+  // a long press should cause opening context menu by default.
+  if ((aTouchEndEvent->mTimeStamp - sSingleTouchStartTimeStamp)
+          .ToMilliseconds() > StaticPrefs::apz_max_tap_time()) {
+    return false;
+  }
+  NS_WARNING_ASSERTION(aTouchEndEvent->mTouches[0]->mChanged,
+                       "The single tap end should be changed");
+  return true;
+}
+
+/* static */
+bool TouchManager::IsPrecedingTouchPointerDownConsumedByContent() {
+  return sPrecedingTouchPointerDownConsumedByContent;
 }
 
 }  // namespace mozilla
