@@ -14,8 +14,6 @@
 #include "nsContentUtils.h"
 #include "nsCSSAnonBoxes.h"
 #include "nsFontMetrics.h"
-#include "nsGfxScrollFrame.h"
-#include "nsIScrollableFrame.h"
 #include "nsLayoutUtils.h"
 #include "nsPresContext.h"
 #include "nsRect.h"
@@ -23,13 +21,14 @@
 #include "nsIFrameInlines.h"
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/Likely.h"
-#include "nsISelection.h"
+#include "mozilla/PresShell.h"
+#include "mozilla/ScrollContainerFrame.h"
+#include "mozilla/dom/Selection.h"
 #include "TextDrawTarget.h"
 
 using mozilla::layout::TextDrawTarget;
 
-namespace mozilla {
-namespace css {
+namespace mozilla::css {
 
 class LazyReferenceRenderingDrawTargetGetterFromFrame final
     : public gfxFontGroup::LazyReferenceDrawTargetGetter {
@@ -39,7 +38,7 @@ class LazyReferenceRenderingDrawTargetGetterFromFrame final
   explicit LazyReferenceRenderingDrawTargetGetterFromFrame(nsIFrame* aFrame)
       : mFrame(aFrame) {}
   virtual already_AddRefed<DrawTarget> GetRefDrawTarget() override {
-    RefPtr<gfxContext> ctx =
+    UniquePtr<gfxContext> ctx =
         mFrame->PresShell()->CreateReferenceRenderingContext();
     RefPtr<DrawTarget> dt = ctx->GetDrawTarget();
     return dt.forget();
@@ -56,12 +55,13 @@ static gfxTextRun* GetEllipsisTextRun(nsIFrame* aFrame) {
       aFrame);
   return fm->GetThebesFontGroup()->GetEllipsisTextRun(
       aFrame->PresContext()->AppUnitsPerDevPixel(),
-      nsLayoutUtils::GetTextRunOrientFlagsForStyle(aFrame->StyleContext()),
+      nsLayoutUtils::GetTextRunOrientFlagsForStyle(aFrame->Style()),
       lazyRefDrawTargetGetter);
 }
 
 static nsIFrame* GetSelfOrNearestBlock(nsIFrame* aFrame) {
-  return nsLayoutUtils::GetAsBlock(aFrame)
+  MOZ_ASSERT(aFrame);
+  return aFrame->IsBlockFrameOrSubclass()
              ? aFrame
              : nsLayoutUtils::FindNearestBlockAncestor(aFrame);
 }
@@ -70,12 +70,11 @@ static nsIFrame* GetSelfOrNearestBlock(nsIFrame* aFrame) {
 // It's not supposed to be called for block frames since we never
 // process block descendants for text-overflow.
 static bool IsAtomicElement(nsIFrame* aFrame, LayoutFrameType aFrameType) {
-  NS_PRECONDITION(
-      !nsLayoutUtils::GetAsBlock(aFrame) || !aFrame->IsBlockOutside(),
-      "unexpected block frame");
-  NS_PRECONDITION(aFrameType != LayoutFrameType::Placeholder,
-                  "unexpected placeholder frame");
-  return !aFrame->IsFrameOfType(nsIFrame::eLineParticipant);
+  MOZ_ASSERT(!aFrame->IsBlockFrameOrSubclass() || !aFrame->IsBlockOutside(),
+             "unexpected block frame");
+  MOZ_ASSERT(aFrameType != LayoutFrameType::Placeholder,
+             "unexpected placeholder frame");
+  return !aFrame->IsLineParticipant();
 }
 
 static bool IsFullyClipped(nsTextFrame* aFrame, nscoord aLeft, nscoord aRight,
@@ -90,11 +89,11 @@ static bool IsFullyClipped(nsTextFrame* aFrame, nscoord aLeft, nscoord aRight,
 }
 
 static bool IsInlineAxisOverflowVisible(nsIFrame* aFrame) {
-  NS_PRECONDITION(nsLayoutUtils::GetAsBlock(aFrame) != nullptr,
-                  "expected a block frame");
+  MOZ_ASSERT(aFrame && aFrame->IsBlockFrameOrSubclass(),
+             "expected a block frame");
 
   nsIFrame* f = aFrame;
-  while (f && f->StyleContext()->IsAnonBox() && !f->IsScrollFrame()) {
+  while (f && f->Style()->IsAnonBox() && !f->IsScrollContainerFrame()) {
     f = f->GetParent();
   }
   if (!f) {
@@ -103,7 +102,7 @@ static bool IsInlineAxisOverflowVisible(nsIFrame* aFrame) {
   auto overflow = aFrame->GetWritingMode().IsVertical()
                       ? f->StyleDisplay()->mOverflowY
                       : f->StyleDisplay()->mOverflowX;
-  return overflow == NS_STYLE_OVERFLOW_VISIBLE;
+  return overflow == StyleOverflow::Visible;
 }
 
 static void ClipMarker(const nsRect& aContentArea, const nsRect& aMarkerRect,
@@ -139,37 +138,32 @@ static bool IsFrameDescendantOfAny(
     nsIFrame* aChild, const TextOverflow::FrameHashtable& aSetOfFrames,
     nsIFrame* aCommonAncestor) {
   for (nsIFrame* f = aChild; f && f != aCommonAncestor;
-       f = nsLayoutUtils::GetCrossDocParentFrame(f)) {
-    if (aSetOfFrames.GetEntry(f)) {
+       f = nsLayoutUtils::GetCrossDocParentFrameInProcess(f)) {
+    if (aSetOfFrames.Contains(f)) {
       return true;
     }
   }
   return false;
 }
 
-class nsDisplayTextOverflowMarker : public nsDisplayItem {
+class nsDisplayTextOverflowMarker final : public nsPaintedDisplayItem {
  public:
   nsDisplayTextOverflowMarker(nsDisplayListBuilder* aBuilder, nsIFrame* aFrame,
                               const nsRect& aRect, nscoord aAscent,
-                              const nsStyleTextOverflowSide* aStyle,
-                              uint32_t aLineNumber, uint32_t aIndex)
-      : nsDisplayItem(aBuilder, aFrame),
+                              const StyleTextOverflowSide& aStyle)
+      : nsPaintedDisplayItem(aBuilder, aFrame),
         mRect(aRect),
         mStyle(aStyle),
-        mAscent(aAscent),
-        mIndex((aLineNumber << 1) + aIndex) {
+        mAscent(aAscent) {
     MOZ_COUNT_CTOR(nsDisplayTextOverflowMarker);
   }
-#ifdef NS_BUILD_REFCNT_LOGGING
-  virtual ~nsDisplayTextOverflowMarker() {
-    MOZ_COUNT_DTOR(nsDisplayTextOverflowMarker);
-  }
-#endif
+
+  MOZ_COUNTED_DTOR_FINAL(nsDisplayTextOverflowMarker)
+
   virtual nsRect GetBounds(nsDisplayListBuilder* aBuilder,
                            bool* aSnap) const override {
     *aSnap = false;
-    nsRect shadowRect = nsLayoutUtils::GetTextShadowRectsUnion(mRect, mFrame);
-    return mRect.Union(shadowRect);
+    return nsLayoutUtils::GetTextShadowRectsUnion(mRect, mFrame);
   }
 
   virtual nsRect GetComponentAlphaBounds(
@@ -188,23 +182,20 @@ class nsDisplayTextOverflowMarker : public nsDisplayItem {
 
   virtual void Paint(nsDisplayListBuilder* aBuilder, gfxContext* aCtx) override;
 
-  virtual uint32_t GetPerFrameKey() const override {
-    return (mIndex << TYPE_BITS) | nsDisplayItem::GetPerFrameKey();
-  }
   void PaintTextToContext(gfxContext* aCtx, nsPoint aOffsetFromRect);
 
   virtual bool CreateWebRenderCommands(
       mozilla::wr::DisplayListBuilder& aBuilder,
       mozilla::wr::IpcResourceUpdateQueue& aResources,
-      const StackingContextHelper& aSc, layers::WebRenderLayerManager* aManager,
+      const StackingContextHelper& aSc,
+      layers::RenderRootStateManager* aManager,
       nsDisplayListBuilder* aDisplayListBuilder) override;
 
   NS_DISPLAY_DECL_NAME("TextOverflow", TYPE_TEXT_OVERFLOW)
  private:
   nsRect mRect;  // in reference frame coordinates
-  const nsStyleTextOverflowSide* mStyle;
+  const StyleTextOverflowSide mStyle;
   nscoord mAscent;  // baseline for the marker text in mRect
-  uint32_t mIndex;
 };
 
 static void PaintTextShadowCallback(gfxContext* aCtx, nsPoint aShadowOffset,
@@ -215,17 +206,14 @@ static void PaintTextShadowCallback(gfxContext* aCtx, nsPoint aShadowOffset,
 
 void nsDisplayTextOverflowMarker::Paint(nsDisplayListBuilder* aBuilder,
                                         gfxContext* aCtx) {
-  DrawTargetAutoDisableSubpixelAntialiasing disable(aCtx->GetDrawTarget(),
-                                                    mDisableSubpixelAA);
-
   nscolor foregroundColor =
-      nsLayoutUtils::GetColor(mFrame, &nsStyleText::mWebkitTextFillColor);
+      nsLayoutUtils::GetTextColor(mFrame, &nsStyleText::mWebkitTextFillColor);
 
   // Paint the text-shadows for the overflow marker
-  nsLayoutUtils::PaintTextShadow(mFrame, aCtx, mRect, mVisibleRect,
-                                 foregroundColor, PaintTextShadowCallback,
-                                 (void*)this);
-  aCtx->SetColor(gfx::Color::FromABGR(foregroundColor));
+  nsLayoutUtils::PaintTextShadow(mFrame, aCtx, mRect,
+                                 GetPaintRect(aBuilder, aCtx), foregroundColor,
+                                 PaintTextShadowCallback, (void*)this);
+  aCtx->SetColor(gfx::sRGBColor::FromABGR(foregroundColor));
   PaintTextToContext(aCtx, nsPoint(0, 0));
 }
 
@@ -247,27 +235,29 @@ void nsDisplayTextOverflowMarker::PaintTextToContext(gfxContext* aCtx,
   }
   pt += aOffsetFromRect;
 
-  if (mStyle->mType == NS_STYLE_TEXT_OVERFLOW_ELLIPSIS) {
+  if (mStyle.IsEllipsis()) {
     gfxTextRun* textRun = GetEllipsisTextRun(mFrame);
     if (textRun) {
       NS_ASSERTION(!textRun->IsRightToLeft(),
                    "Ellipsis textruns should always be LTR!");
       gfx::Point gfxPt(pt.x, pt.y);
+      auto& paletteCache = mFrame->PresContext()->FontPaletteCache();
       textRun->Draw(gfxTextRun::Range(textRun), gfxPt,
-                    gfxTextRun::DrawParams(aCtx));
+                    gfxTextRun::DrawParams(aCtx, paletteCache));
     }
   } else {
     RefPtr<nsFontMetrics> fm =
         nsLayoutUtils::GetInflatedFontMetricsForFrame(mFrame);
-    nsLayoutUtils::DrawString(mFrame, *fm, aCtx, mStyle->mString.get(),
-                              mStyle->mString.Length(), pt);
+    nsDependentAtomString str16(mStyle.AsString().AsAtom());
+    nsLayoutUtils::DrawString(mFrame, *fm, aCtx, str16.get(), str16.Length(),
+                              pt);
   }
 }
 
 bool nsDisplayTextOverflowMarker::CreateWebRenderCommands(
     mozilla::wr::DisplayListBuilder& aBuilder,
     mozilla::wr::IpcResourceUpdateQueue& aResources,
-    const StackingContextHelper& aSc, layers::WebRenderLayerManager* aManager,
+    const StackingContextHelper& aSc, layers::RenderRootStateManager* aManager,
     nsDisplayListBuilder* aDisplayListBuilder) {
   bool snap;
   nsRect bounds = GetBounds(aDisplayListBuilder, &snap);
@@ -277,46 +267,39 @@ bool nsDisplayTextOverflowMarker::CreateWebRenderCommands(
 
   // Run the rendering algorithm to capture the glyphs and shadows
   RefPtr<TextDrawTarget> textDrawer =
-      new TextDrawTarget(aBuilder, aSc, aManager, this, bounds);
-  RefPtr<gfxContext> captureCtx = gfxContext::CreateOrNull(textDrawer);
-  Paint(aDisplayListBuilder, captureCtx);
+      new TextDrawTarget(aBuilder, aResources, aSc, aManager, this, bounds);
+  MOZ_ASSERT(textDrawer->IsValid());
+  if (!textDrawer->IsValid()) {
+    return false;
+  }
+  gfxContext captureCtx(textDrawer);
+  Paint(aDisplayListBuilder, &captureCtx);
   textDrawer->TerminateShadows();
 
-  return !textDrawer->HasUnsupportedFeatures();
+  return textDrawer->Finish();
 }
 
 TextOverflow::TextOverflow(nsDisplayListBuilder* aBuilder,
-                           nsIFrame* aBlockFrame)
+                           nsBlockFrame* aBlockFrame)
     : mContentArea(aBlockFrame->GetWritingMode(),
                    aBlockFrame->GetContentRectRelativeToSelf(),
                    aBlockFrame->GetSize()),
       mBuilder(aBuilder),
       mBlock(aBlockFrame),
-      mScrollableFrame(nsLayoutUtils::GetScrollableFrameFor(aBlockFrame)),
+      mScrollContainerFrame(
+          nsLayoutUtils::GetScrollContainerFrameFor(aBlockFrame)),
+      mMarkerList(aBuilder),
       mBlockSize(aBlockFrame->GetSize()),
       mBlockWM(aBlockFrame->GetWritingMode()),
+      mCanHaveInlineAxisScrollbar(false),
+      mInLineClampContext(aBlockFrame->IsInLineClampContext()),
       mAdjustForPixelSnapping(false) {
-#ifdef MOZ_XUL
-  if (!mScrollableFrame) {
-    nsAtom* pseudoType = aBlockFrame->StyleContext()->GetPseudo();
-    if (pseudoType == nsCSSAnonBoxes::mozXULAnonymousBlock) {
-      mScrollableFrame =
-          nsLayoutUtils::GetScrollableFrameFor(aBlockFrame->GetParent());
-      // nsXULScrollFrame::ClampAndSetBounds rounds to nearest pixels
-      // for RTL blocks (also for overflow:hidden), so we need to move
-      // the edges 1px outward in ExamineLineFrames to avoid triggering
-      // a text-overflow marker in this case.
-      mAdjustForPixelSnapping = !mBlockWM.IsBidiLTR();
-    }
-  }
-#endif
-  mCanHaveInlineAxisScrollbar = false;
-  if (mScrollableFrame) {
+  if (mScrollContainerFrame) {
     auto scrollbarStyle =
         mBlockWM.IsVertical()
-            ? mScrollableFrame->GetScrollbarStyles().mVertical
-            : mScrollableFrame->GetScrollbarStyles().mHorizontal;
-    mCanHaveInlineAxisScrollbar = scrollbarStyle != NS_STYLE_OVERFLOW_HIDDEN;
+            ? mScrollContainerFrame->GetScrollStyles().mVertical
+            : mScrollContainerFrame->GetScrollStyles().mHorizontal;
+    mCanHaveInlineAxisScrollbar = scrollbarStyle != StyleOverflow::Hidden;
     if (!mAdjustForPixelSnapping) {
       // Scrolling to the end position can leave some text still overflowing due
       // to pixel snapping behaviour in our scrolling code.
@@ -325,36 +308,48 @@ TextOverflow::TextOverflow(nsDisplayListBuilder* aBuilder,
     // Use a null containerSize to convert a vector from logical to physical.
     const nsSize nullContainerSize;
     mContentArea.MoveBy(
-        mBlockWM, LogicalPoint(mBlockWM, mScrollableFrame->GetScrollPosition(),
-                               nullContainerSize));
+        mBlockWM,
+        LogicalPoint(mBlockWM, mScrollContainerFrame->GetScrollPosition(),
+                     nullContainerSize));
   }
-  uint8_t direction = aBlockFrame->StyleVisibility()->mDirection;
+  StyleDirection direction = aBlockFrame->StyleVisibility()->mDirection;
   const nsStyleTextReset* style = aBlockFrame->StyleTextReset();
+
+  const auto& textOverflow = style->mTextOverflow;
+  bool shouldToggleDirection =
+      textOverflow.sides_are_logical && (direction == StyleDirection::Rtl);
+  const auto& leftSide =
+      shouldToggleDirection ? textOverflow.second : textOverflow.first;
+  const auto& rightSide =
+      shouldToggleDirection ? textOverflow.first : textOverflow.second;
+
   if (mBlockWM.IsBidiLTR()) {
-    mIStart.Init(style->mTextOverflow.GetLeft(direction));
-    mIEnd.Init(style->mTextOverflow.GetRight(direction));
+    mIStart.Init(leftSide);
+    mIEnd.Init(rightSide);
   } else {
-    mIStart.Init(style->mTextOverflow.GetRight(direction));
-    mIEnd.Init(style->mTextOverflow.GetLeft(direction));
+    mIStart.Init(rightSide);
+    mIEnd.Init(leftSide);
   }
   // The left/right marker string is setup in ExamineLineFrames when a line
   // has overflow on that side.
 }
 
-/* static */ UniquePtr<TextOverflow> TextOverflow::WillProcessLines(
-    nsDisplayListBuilder* aBuilder, nsIFrame* aBlockFrame) {
-  // Ignore 'text-overflow' for event and frame visibility processing.
+/* static */
+Maybe<TextOverflow> TextOverflow::WillProcessLines(
+    nsDisplayListBuilder* aBuilder, nsBlockFrame* aBlockFrame) {
+  // Ignore text-overflow and -webkit-line-clamp for event and frame visibility
+  // processing.
   if (aBuilder->IsForEventDelivery() || aBuilder->IsForFrameVisibility() ||
-      !CanHaveTextOverflow(aBlockFrame)) {
-    return nullptr;
+      !CanHaveOverflowMarkers(aBlockFrame)) {
+    return Nothing();
   }
-  nsIScrollableFrame* scrollableFrame =
-      nsLayoutUtils::GetScrollableFrameFor(aBlockFrame);
-  if (scrollableFrame && scrollableFrame->IsTransformingByAPZ()) {
+  ScrollContainerFrame* scrollContainerFrame =
+      nsLayoutUtils::GetScrollContainerFrameFor(aBlockFrame);
+  if (scrollContainerFrame && scrollContainerFrame->IsTransformingByAPZ()) {
     // If the APZ is actively scrolling this, don't bother with markers.
-    return nullptr;
+    return Nothing();
   }
-  return MakeUnique<TextOverflow>(aBuilder, aBlockFrame);
+  return Some(TextOverflow(aBuilder, aBlockFrame));
 }
 
 void TextOverflow::ExamineFrameSubtree(nsIFrame* aFrame,
@@ -384,7 +379,7 @@ void TextOverflow::ExamineFrameSubtree(nsIFrame* aFrame,
     }
     if (isAtomic && ((mIStart.mActive && overflowIStart) ||
                      (mIEnd.mActive && overflowIEnd))) {
-      aFramesToHide->PutEntry(aFrame);
+      aFramesToHide->Insert(aFrame);
     } else if (isAtomic || frameType == LayoutFrameType::Text) {
       AnalyzeMarkerEdges(aFrame, frameType, aInsideMarkersArea, aFramesToHide,
                          aAlignmentEdges, aFoundVisibleTextOrAtomic,
@@ -460,21 +455,24 @@ void TextOverflow::AnalyzeMarkerEdges(nsIFrame* aFrame,
           if (iendOverlap > 0) {
             snappedRect.ISize(mBlockWM) -= snappedIEnd;
           }
-          aAlignmentEdges->Accumulate(mBlockWM, snappedRect);
+          aAlignmentEdges->AccumulateInner(mBlockWM, snappedRect);
           *aFoundVisibleTextOrAtomic = true;
         }
       }
     } else {
-      aFramesToHide->PutEntry(aFrame);
+      aFramesToHide->Insert(aFrame);
     }
   } else if (!insideIStartEdge || !insideIEndEdge) {
     // frame is outside
+    if (!insideIStartEdge) {
+      aAlignmentEdges->AccumulateOuter(mBlockWM, borderRect);
+    }
     if (IsAtomicElement(aFrame, aFrameType)) {
-      aFramesToHide->PutEntry(aFrame);
+      aFramesToHide->Insert(aFrame);
     }
   } else {
     // frame is inside
-    aAlignmentEdges->Accumulate(mBlockWM, borderRect);
+    aAlignmentEdges->AccumulateInner(mBlockWM, borderRect);
     if (aFrameType == LayoutFrameType::Text) {
       auto textFrame = static_cast<nsTextFrame*>(aFrame);
       if (textFrame->HasNonSuppressedText()) {
@@ -490,12 +488,12 @@ LogicalRect TextOverflow::ExamineLineFrames(nsLineBox* aLine,
                                             FrameHashtable* aFramesToHide,
                                             AlignmentEdges* aAlignmentEdges) {
   // No ellipsing for 'clip' style.
-  bool suppressIStart = mIStart.mStyle->mType == NS_STYLE_TEXT_OVERFLOW_CLIP;
-  bool suppressIEnd = mIEnd.mStyle->mType == NS_STYLE_TEXT_OVERFLOW_CLIP;
+  bool suppressIStart = mIStart.IsSuppressed(mInLineClampContext);
+  bool suppressIEnd = mIEnd.IsSuppressed(mInLineClampContext);
   if (mCanHaveInlineAxisScrollbar) {
-    LogicalPoint pos(mBlockWM, mScrollableFrame->GetScrollPosition(),
+    LogicalPoint pos(mBlockWM, mScrollContainerFrame->GetScrollPosition(),
                      mBlockSize);
-    LogicalRect scrollRange(mBlockWM, mScrollableFrame->GetScrollRange(),
+    LogicalRect scrollRange(mBlockWM, mScrollContainerFrame->GetScrollRange(),
                             mBlockSize);
     // No ellipsing when nothing to scroll to on that side (this includes
     // overflow:auto that doesn't trigger a horizontal scrollbar).
@@ -503,7 +501,10 @@ LogicalRect TextOverflow::ExamineLineFrames(nsLineBox* aLine,
       suppressIStart = true;
     }
     if (pos.I(mBlockWM) >= scrollRange.IEnd(mBlockWM)) {
-      suppressIEnd = true;
+      // Except that we always want to display a -webkit-line-clamp ellipsis.
+      if (!mIEnd.mHasBlockEllipsis) {
+        suppressIEnd = true;
+      }
     }
   }
 
@@ -540,24 +541,30 @@ LogicalRect TextOverflow::ExamineLineFrames(nsLineBox* aLine,
     }
   }
 
-  LogicalRect lineRect(mBlockWM, aLine->GetScrollableOverflowArea(),
-                       mBlockSize);
-  const bool istartOverflow =
+  LogicalRect lineRect(mBlockWM, aLine->ScrollableOverflowRect(), mBlockSize);
+  const bool istartWantsMarker =
       !suppressIStart &&
       lineRect.IStart(mBlockWM) < contentArea.IStart(mBlockWM);
-  const bool iendOverflow =
+  const bool iendWantsTextOverflowMarker =
       !suppressIEnd && lineRect.IEnd(mBlockWM) > contentArea.IEnd(mBlockWM);
-  if (!istartOverflow && !iendOverflow) {
-    // The line does not overflow on a side we should ellipsize.
+  const bool iendWantsBlockEllipsisMarker =
+      !suppressIEnd && mIEnd.mHasBlockEllipsis;
+  const bool iendWantsMarker =
+      iendWantsTextOverflowMarker || iendWantsBlockEllipsisMarker;
+  if (!istartWantsMarker && !iendWantsMarker) {
+    // We don't need any markers on this line.
     return nonSnappedContentArea;
   }
 
   int pass = 0;
   bool retryEmptyLine = true;
-  bool guessIStart = istartOverflow;
-  bool guessIEnd = iendOverflow;
-  mIStart.mActive = istartOverflow;
-  mIEnd.mActive = iendOverflow;
+  bool guessIStart = istartWantsMarker;
+  bool guessIEnd = iendWantsMarker;
+  mIStart.mActive = istartWantsMarker;
+  mIEnd.mActive = iendWantsMarker;
+  mIStart.mEdgeAligned = mCanHaveInlineAxisScrollbar && istartWantsMarker;
+  mIEnd.mEdgeAligned =
+      mCanHaveInlineAxisScrollbar && iendWantsTextOverflowMarker;
   bool clippedIStartMarker = false;
   bool clippedIEndMarker = false;
   do {
@@ -600,7 +607,8 @@ LogicalRect TextOverflow::ExamineLineFrames(nsLineBox* aLine,
                           &clippedMarkerEdges);
     }
     if (!foundVisibleTextOrAtomic && retryEmptyLine) {
-      aAlignmentEdges->mAssigned = false;
+      aAlignmentEdges->mAssignedInner = false;
+      aAlignmentEdges->mIEndOuter = 0;
       aFramesToHide->Clear();
       pass = -1;
       if (mIStart.IsNeeded() && mIStart.mActive && !clippedIStartMarker) {
@@ -634,9 +642,23 @@ LogicalRect TextOverflow::ExamineLineFrames(nsLineBox* aLine,
       // so examine the line again without suppressing markers.
       retryEmptyLine = false;
       mIStart.mISize = mIStart.mIntrinsicISize;
-      mIStart.mActive = guessIStart = istartOverflow;
+      mIStart.mActive = guessIStart = istartWantsMarker;
       mIEnd.mISize = mIEnd.mIntrinsicISize;
-      mIEnd.mActive = guessIEnd = iendOverflow;
+      mIEnd.mActive = guessIEnd = iendWantsMarker;
+      // If we wanted to place a block ellipsis but didn't, due to not having
+      // any visible content to align to or the line's content being scrolled
+      // out of view, then clip the ellipsis so that it looks like it is aligned
+      // with the out of view content.
+      if (mIEnd.IsNeeded() && mIEnd.mActive && mIEnd.mHasBlockEllipsis) {
+        NS_ASSERTION(nonSnappedContentArea.IStart(mBlockWM) >
+                         aAlignmentEdges->mIEndOuter,
+                     "Expected the alignment edge for the out of view content "
+                     "to be before the start of the content area");
+        mIEnd.mISize = std::max(
+            mIEnd.mIntrinsicISize - (nonSnappedContentArea.IStart(mBlockWM) -
+                                     aAlignmentEdges->mIEndOuter),
+            0);
+      }
       continue;
     }
     if (guessIStart == (mIStart.mActive && mIStart.IsNeeded()) &&
@@ -651,10 +673,10 @@ LogicalRect TextOverflow::ExamineLineFrames(nsLineBox* aLine,
     }
     NS_ASSERTION(pass == 0, "2nd pass should never guess wrong");
   } while (++pass != 2);
-  if (!istartOverflow || !mIStart.mActive) {
+  if (!istartWantsMarker || !mIStart.mActive) {
     mIStart.Reset();
   }
-  if (!iendOverflow || !mIEnd.mActive) {
+  if (!iendWantsMarker || !mIEnd.mActive) {
     mIEnd.Reset();
   }
   return nonSnappedContentArea;
@@ -662,13 +684,16 @@ LogicalRect TextOverflow::ExamineLineFrames(nsLineBox* aLine,
 
 void TextOverflow::ProcessLine(const nsDisplayListSet& aLists, nsLineBox* aLine,
                                uint32_t aLineNumber) {
-  NS_ASSERTION(mIStart.mStyle->mType != NS_STYLE_TEXT_OVERFLOW_CLIP ||
-                   mIEnd.mStyle->mType != NS_STYLE_TEXT_OVERFLOW_CLIP,
-               "TextOverflow with 'clip' for both sides");
+  if (mIStart.mStyle->IsClip() && mIEnd.mStyle->IsClip() &&
+      !aLine->HasLineClampEllipsis()) {
+    return;
+  }
+
   mIStart.Reset();
-  mIStart.mActive = mIStart.mStyle->mType != NS_STYLE_TEXT_OVERFLOW_CLIP;
+  mIStart.mActive = !mIStart.mStyle->IsClip();
   mIEnd.Reset();
-  mIEnd.mActive = mIEnd.mStyle->mType != NS_STYLE_TEXT_OVERFLOW_CLIP;
+  mIEnd.mHasBlockEllipsis = aLine->HasLineClampEllipsis();
+  mIEnd.mActive = !mIEnd.IsSuppressed(mInLineClampContext);
 
   FrameHashtable framesToHide(64);
   AlignmentEdges alignmentEdges;
@@ -679,11 +704,10 @@ void TextOverflow::ProcessLine(const nsDisplayListSet& aLists, nsLineBox* aLine,
   if (!needIStart && !needIEnd) {
     return;
   }
-  NS_ASSERTION(
-      mIStart.mStyle->mType != NS_STYLE_TEXT_OVERFLOW_CLIP || !needIStart,
-      "left marker for 'clip'");
-  NS_ASSERTION(mIEnd.mStyle->mType != NS_STYLE_TEXT_OVERFLOW_CLIP || !needIEnd,
-               "right marker for 'clip'");
+  NS_ASSERTION(!mIStart.IsSuppressed(mInLineClampContext) || !needIStart,
+               "left marker when not needed");
+  NS_ASSERTION(!mIEnd.IsSuppressed(mInLineClampContext) || !needIEnd,
+               "right marker when not needed");
 
   // If there is insufficient space for both markers then keep the one on the
   // end side per the block's 'direction'.
@@ -698,16 +722,35 @@ void TextOverflow::ProcessLine(const nsDisplayListSet& aLists, nsLineBox* aLine,
   if (needIEnd) {
     InflateIEnd(mBlockWM, &insideMarkersArea, -mIEnd.mISize);
   }
-  if (!mCanHaveInlineAxisScrollbar && alignmentEdges.mAssigned) {
+
+  if (alignmentEdges.mAssignedInner) {
+    if (mIStart.mEdgeAligned) {
+      alignmentEdges.mIStart = insideMarkersArea.IStart(mBlockWM);
+    }
+    if (mIEnd.mEdgeAligned) {
+      alignmentEdges.mIEnd = insideMarkersArea.IEnd(mBlockWM);
+    }
     LogicalRect alignmentRect(mBlockWM, alignmentEdges.mIStart,
                               insideMarkersArea.BStart(mBlockWM),
                               alignmentEdges.ISize(), 1);
     insideMarkersArea.IntersectRect(insideMarkersArea, alignmentRect);
+  } else {
+    // There was no content on the line that was visible at the current scolled
+    // position.  If we wanted to place a block ellipsis but failed due to
+    // having no visible content to align it to, we still need to ensure it
+    // is displayed.  It goes at the start of the line, even though it's an
+    // IEnd marker, since that is the side of the line that the content has
+    // been scrolled past.  We set the insideMarkersArea to a zero-sized
+    // rectangle placed next to the scrolled-out-of-view content.
+    if (mIEnd.mHasBlockEllipsis) {
+      insideMarkersArea = LogicalRect(mBlockWM, alignmentEdges.mIEndOuter,
+                                      insideMarkersArea.BStart(mBlockWM), 0, 1);
+    }
   }
 
   // Clip and remove display items as needed at the final marker edges.
   nsDisplayList* lists[] = {aLists.Content(), aLists.PositionedDescendants()};
-  for (uint32_t i = 0; i < ArrayLength(lists); ++i) {
+  for (uint32_t i = 0; i < std::size(lists); ++i) {
     PruneDisplayListContents(lists[i], framesToHide, insideMarkersArea);
   }
   CreateMarkers(aLine, needIStart, needIEnd, insideMarkersArea, contentArea,
@@ -717,9 +760,7 @@ void TextOverflow::ProcessLine(const nsDisplayListSet& aLists, nsLineBox* aLine,
 void TextOverflow::PruneDisplayListContents(
     nsDisplayList* aList, const FrameHashtable& aFramesToHide,
     const LogicalRect& aInsideMarkersArea) {
-  nsDisplayList saved;
-  nsDisplayItem* item;
-  while ((item = aList->RemoveBottom())) {
+  for (nsDisplayItem* item : aList->TakeItems()) {
     nsIFrame* itemFrame = item->Frame();
     if (IsFrameDescendantOfAny(itemFrame, aFramesToHide, mBlock)) {
       item->Destroy(mBuilder);
@@ -733,62 +774,83 @@ void TextOverflow::PruneDisplayListContents(
       }
     }
 
-    nsCharClipDisplayItem* charClip =
-        itemFrame ? nsCharClipDisplayItem::CheckCast(item) : nullptr;
-    if (charClip && GetSelfOrNearestBlock(itemFrame) == mBlock) {
+    nsDisplayText* textItem =
+        itemFrame ? nsDisplayText::CheckCast(item) : nullptr;
+    if (textItem && GetSelfOrNearestBlock(itemFrame) == mBlock) {
       LogicalRect rect =
           GetLogicalScrollableOverflowRectRelativeToBlock(itemFrame);
       if (mIStart.IsNeeded()) {
         nscoord istart =
             aInsideMarkersArea.IStart(mBlockWM) - rect.IStart(mBlockWM);
         if (istart > 0) {
-          (mBlockWM.IsBidiLTR() ? charClip->mVisIStartEdge
-                                : charClip->mVisIEndEdge) = istart;
+          (mBlockWM.IsBidiLTR() ? textItem->VisIStartEdge()
+                                : textItem->VisIEndEdge()) = istart;
         }
       }
       if (mIEnd.IsNeeded()) {
         nscoord iend = rect.IEnd(mBlockWM) - aInsideMarkersArea.IEnd(mBlockWM);
         if (iend > 0) {
-          (mBlockWM.IsBidiLTR() ? charClip->mVisIEndEdge
-                                : charClip->mVisIStartEdge) = iend;
+          (mBlockWM.IsBidiLTR() ? textItem->VisIEndEdge()
+                                : textItem->VisIStartEdge()) = iend;
         }
       }
     }
 
-    saved.AppendToTop(item);
+    aList->AppendToTop(item);
   }
-  aList->AppendToTop(&saved);
 }
 
-/* static */ bool TextOverflow::HasClippedOverflow(nsIFrame* aBlockFrame) {
+/* static */
+bool TextOverflow::HasClippedTextOverflow(nsIFrame* aBlockFrame) {
   const nsStyleTextReset* style = aBlockFrame->StyleTextReset();
-  return style->mTextOverflow.mLeft.mType == NS_STYLE_TEXT_OVERFLOW_CLIP &&
-         style->mTextOverflow.mRight.mType == NS_STYLE_TEXT_OVERFLOW_CLIP;
+  return style->mTextOverflow.first.IsClip() &&
+         style->mTextOverflow.second.IsClip();
 }
 
-/* static */ bool TextOverflow::CanHaveTextOverflow(nsIFrame* aBlockFrame) {
+/* static */
+bool TextOverflow::HasBlockEllipsis(nsIFrame* aBlockFrame) {
+  nsBlockFrame* f = do_QueryFrame(aBlockFrame);
+  return f && f->HasLineClampEllipsis();
+}
+
+static bool BlockCanHaveLineClampEllipsis(nsBlockFrame* aBlockFrame,
+                                          bool aBeforeReflow) {
+  if (aBeforeReflow) {
+    return aBlockFrame->IsInLineClampContext();
+  }
+  return aBlockFrame->HasLineClampEllipsis();
+}
+
+/* static */
+bool TextOverflow::CanHaveOverflowMarkers(nsBlockFrame* aBlockFrame,
+                                          BeforeReflow aBeforeReflow) {
+  if (BlockCanHaveLineClampEllipsis(aBlockFrame,
+                                    aBeforeReflow == BeforeReflow::Yes)) {
+    return true;
+  }
+
   // Nothing to do for text-overflow:clip or if 'overflow-x/y:visible'.
-  if (HasClippedOverflow(aBlockFrame) ||
+  if (HasClippedTextOverflow(aBlockFrame) ||
       IsInlineAxisOverflowVisible(aBlockFrame)) {
     return false;
   }
 
-  // Skip ComboboxControlFrame because it would clip the drop-down arrow.
-  // Its anon block inherits 'text-overflow' and does what is expected.
-  if (aBlockFrame->IsComboboxControlFrame()) {
+  // Skip the combobox anonymous block because it would clip the drop-down
+  // arrow. The inner label inherits 'text-overflow' and does the right thing.
+  if (aBlockFrame->GetParent() &&
+      aBlockFrame->GetParent()->IsComboboxControlFrame()) {
     return false;
   }
 
   // Inhibit the markers if a descendant content owns the caret.
   RefPtr<nsCaret> caret = aBlockFrame->PresShell()->GetCaret();
   if (caret && caret->IsVisible()) {
-    nsCOMPtr<nsISelection> domSelection = caret->GetSelection();
+    RefPtr<dom::Selection> domSelection = caret->GetSelection();
     if (domSelection) {
-      nsCOMPtr<nsIDOMNode> node;
-      domSelection->GetFocusNode(getter_AddRefs(node));
-      nsCOMPtr<nsIContent> content = do_QueryInterface(node);
-      if (content && nsContentUtils::ContentIsDescendantOf(
-                         content, aBlockFrame->GetContent())) {
+      nsCOMPtr<nsIContent> content =
+          nsIContent::FromNodeOrNull(domSelection->GetFocusNode());
+      if (content &&
+          content->IsInclusiveDescendantOf(aBlockFrame->GetContent())) {
         return false;
       }
     }
@@ -801,6 +863,10 @@ void TextOverflow::CreateMarkers(const nsLineBox* aLine, bool aCreateIStart,
                                  const LogicalRect& aInsideMarkersArea,
                                  const LogicalRect& aContentArea,
                                  uint32_t aLineNumber) {
+  if (!mBlock->IsVisibleForPainting()) {
+    return;
+  }
+
   if (aCreateIStart) {
     DisplayListClipState::AutoSaveRestore clipState(mBuilder);
 
@@ -812,10 +878,10 @@ void TextOverflow::CreateMarkers(const nsLineBox* aLine, bool aCreateIStart,
         markerLogicalRect.GetPhysicalRect(mBlockWM, mBlockSize) + offset;
     ClipMarker(aContentArea.GetPhysicalRect(mBlockWM, mBlockSize) + offset,
                markerRect, clipState);
-    nsDisplayItem* marker = MakeDisplayItem<nsDisplayTextOverflowMarker>(
-        mBuilder, mBlock, markerRect, aLine->GetLogicalAscent(), mIStart.mStyle,
-        aLineNumber, 0);
-    mMarkerList.AppendToTop(marker);
+
+    mMarkerList.AppendNewToTopWithIndex<nsDisplayTextOverflowMarker>(
+        mBuilder, mBlock, /* aIndex = */ (aLineNumber << 1) + 0, markerRect,
+        aLine->GetLogicalAscent(), *mIStart.mStyle);
   }
 
   if (aCreateIEnd) {
@@ -829,10 +895,12 @@ void TextOverflow::CreateMarkers(const nsLineBox* aLine, bool aCreateIStart,
         markerLogicalRect.GetPhysicalRect(mBlockWM, mBlockSize) + offset;
     ClipMarker(aContentArea.GetPhysicalRect(mBlockWM, mBlockSize) + offset,
                markerRect, clipState);
-    nsDisplayItem* marker = MakeDisplayItem<nsDisplayTextOverflowMarker>(
-        mBuilder, mBlock, markerRect, aLine->GetLogicalAscent(), mIEnd.mStyle,
-        aLineNumber, 1);
-    mMarkerList.AppendToTop(marker);
+
+    mMarkerList.AppendNewToTopWithIndex<nsDisplayTextOverflowMarker>(
+        mBuilder, mBlock, /* aIndex = */ (aLineNumber << 1) + 1, markerRect,
+        aLine->GetLogicalAscent(),
+        mIEnd.mHasBlockEllipsis ? StyleTextOverflowSide::Ellipsis()
+                                : *mIEnd.mStyle);
   }
 }
 
@@ -841,7 +909,12 @@ void TextOverflow::Marker::SetupString(nsIFrame* aFrame) {
     return;
   }
 
-  if (mStyle->mType == NS_STYLE_TEXT_OVERFLOW_ELLIPSIS) {
+  // A limitation here is that at the IEnd of a line, we only ever render one of
+  // a text-overflow marker and a -webkit-line-clamp block ellipsis.  Since we
+  // don't track the block ellipsis string and the text-overflow marker string
+  // separately, if both apply to the element, we will always use "…" as the
+  // string for text-overflow.
+  if (HasBlockEllipsis(aFrame) || mStyle->IsEllipsis()) {
     gfxTextRun* textRun = GetEllipsisTextRun(aFrame);
     if (textRun) {
       mISize = textRun->GetAdvanceWidth();
@@ -849,16 +922,15 @@ void TextOverflow::Marker::SetupString(nsIFrame* aFrame) {
       mISize = 0;
     }
   } else {
-    RefPtr<gfxContext> rc =
+    UniquePtr<gfxContext> rc =
         aFrame->PresShell()->CreateReferenceRenderingContext();
     RefPtr<nsFontMetrics> fm =
         nsLayoutUtils::GetInflatedFontMetricsForFrame(aFrame);
-    mISize = nsLayoutUtils::AppUnitWidthOfStringBidi(mStyle->mString, aFrame,
-                                                     *fm, *rc);
+    mISize = nsLayoutUtils::AppUnitWidthOfStringBidi(
+        nsDependentAtomString(mStyle->AsString().AsAtom()), aFrame, *fm, *rc);
   }
   mIntrinsicISize = mISize;
   mInitialized = true;
 }
 
-}  // namespace css
-}  // namespace mozilla
+}  // namespace mozilla::css
