@@ -15,13 +15,13 @@
 #include "mozilla/PresShell.h"
 #include "mozilla/ProfilerLabels.h"
 #include "mozilla/RestyleManager.h"
+#include "mozilla/ScrollContainerFrame.h"
 #include "nsGkAtoms.h"
-#include "nsIScrollableFrame.h"
+#include "mozilla/dom/ViewTransition.h"
 #include "nsAbsoluteContainingBlock.h"
 #include "nsCanvasFrame.h"
 #include "nsLayoutUtils.h"
 #include "nsSubDocumentFrame.h"
-#include "nsIMozBrowserFrame.h"
 #include "nsPlaceholderFrame.h"
 #include "MobileViewportManager.h"
 
@@ -67,7 +67,7 @@ void ViewportFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
 
   // If we have a scrollframe then it takes care of creating the display list
   // for the top layer, but otherwise we need to do it here.
-  if (!kid->IsScrollFrame()) {
+  if (!kid->IsScrollContainerFrame()) {
     bool isOpaque = false;
     if (auto* list = BuildDisplayListForTopLayer(aBuilder, &isOpaque)) {
       if (isOpaque) {
@@ -81,19 +81,11 @@ void ViewportFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
 }
 
 #ifdef DEBUG
-/**
- * Returns whether we are going to put an element in the top layer for
- * fullscreen. This function should matches the CSS rule in ua.css.
- */
+// Returns whether we are going to put an element in the top layer for
+// fullscreen. This function should matches the CSS rules in ua.css and xul.css.
 static bool ShouldInTopLayerForFullscreen(dom::Element* aElement) {
-  if (!aElement->GetParent()) {
-    return false;
-  }
-  nsCOMPtr<nsIMozBrowserFrame> browserFrame = do_QueryInterface(aElement);
-  if (browserFrame && browserFrame->GetReallyIsBrowser()) {
-    return false;
-  }
-  return true;
+  return !aElement->IsRootElement() &&
+         !aElement->IsXULElement(nsGkAtoms::browser);
 }
 #endif  // DEBUG
 
@@ -171,7 +163,9 @@ nsDisplayWrapList* ViewportFrame::BuildDisplayListForTopLayer(
     nsDisplayListBuilder* aBuilder, bool* aIsOpaque) {
   nsDisplayList topLayerList(aBuilder);
 
-  nsTArray<dom::Element*> topLayer = PresContext()->Document()->GetTopLayer();
+  auto* doc = PresContext()->Document();
+
+  nsTArray<dom::Element*> topLayer = doc->GetTopLayer();
   for (dom::Element* elem : topLayer) {
     nsIFrame* frame = elem->GetPrimaryFrame();
     if (!frame) {
@@ -223,6 +217,17 @@ nsDisplayWrapList* ViewportFrame::BuildDisplayListForTopLayer(
     BuildDisplayListForTopLayerFrame(aBuilder, frame, &topLayerList);
   }
 
+  if (dom::ViewTransition* vt = doc->GetActiveViewTransition()) {
+    if (dom::Element* root = vt->GetRoot()) {
+      if (nsIFrame* frame = root->GetPrimaryFrame()) {
+        MOZ_ASSERT(frame->StyleDisplay()->mTopLayer != StyleTopLayer::None,
+                   "ua.css should ensure this");
+        MOZ_ASSERT(frame->HasAnyStateBits(NS_FRAME_OUT_OF_FLOW));
+        BuildDisplayListForTopLayerFrame(aBuilder, frame, &topLayerList);
+      }
+    }
+  }
+
   if (nsCanvasFrame* canvasFrame = PresShell()->GetCanvasFrame()) {
     if (dom::Element* container = canvasFrame->GetCustomContentContainer()) {
       if (nsIFrame* frame = container->GetPrimaryFrame()) {
@@ -270,45 +275,32 @@ void ViewportFrame::InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
                                  std::move(aFrameList));
 }
 
-void ViewportFrame::RemoveFrame(ChildListID aListID, nsIFrame* aOldFrame) {
+void ViewportFrame::RemoveFrame(DestroyContext& aContext, ChildListID aListID,
+                                nsIFrame* aOldFrame) {
   NS_ASSERTION(aListID == FrameChildListID::Principal, "unexpected child list");
-  nsContainerFrame::RemoveFrame(aListID, aOldFrame);
+  nsContainerFrame::RemoveFrame(aContext, aListID, aOldFrame);
 }
 #endif
 
-/* virtual */
-nscoord ViewportFrame::GetMinISize(gfxContext* aRenderingContext) {
-  nscoord result;
-  DISPLAY_MIN_INLINE_SIZE(this, result);
-  if (mFrames.IsEmpty())
-    result = 0;
-  else
-    result = mFrames.FirstChild()->GetMinISize(aRenderingContext);
-
-  return result;
-}
-
-/* virtual */
-nscoord ViewportFrame::GetPrefISize(gfxContext* aRenderingContext) {
-  nscoord result;
-  DISPLAY_PREF_INLINE_SIZE(this, result);
-  if (mFrames.IsEmpty())
-    result = 0;
-  else
-    result = mFrames.FirstChild()->GetPrefISize(aRenderingContext);
-
-  return result;
+nscoord ViewportFrame::IntrinsicISize(const IntrinsicSizeInput& aInput,
+                                      IntrinsicISizeType aType) {
+  return mFrames.IsEmpty()
+             ? 0
+             : mFrames.FirstChild()->IntrinsicISize(aInput, aType);
 }
 
 nsPoint ViewportFrame::AdjustReflowInputForScrollbars(
     ReflowInput* aReflowInput) const {
   // Get our prinicpal child frame and see if we're scrollable
   nsIFrame* kidFrame = mFrames.FirstChild();
-  nsIScrollableFrame* scrollingFrame = do_QueryFrame(kidFrame);
 
-  if (scrollingFrame) {
+  if (ScrollContainerFrame* scrollContainerFrame = do_QueryFrame(kidFrame)) {
+    // Note: In ReflowInput::CalculateHypotheticalPosition(), we exclude the
+    // scrollbar or scrollbar-gutter area when computing the offset to
+    // ViewportFrame. Ensure the code there remains in sync with the logic here.
     WritingMode wm = aReflowInput->GetWritingMode();
-    LogicalMargin scrollbars(wm, scrollingFrame->GetActualScrollbarSizes());
+    LogicalMargin scrollbars(wm,
+                             scrollContainerFrame->GetActualScrollbarSizes());
     aReflowInput->SetComputedISize(
         aReflowInput->ComputedISize() - scrollbars.IStartEnd(wm),
         ReflowInput::ResetResizeFlags::No);
@@ -324,19 +316,8 @@ nsPoint ViewportFrame::AdjustReflowInputForScrollbars(
 
 nsRect ViewportFrame::AdjustReflowInputAsContainingBlock(
     ReflowInput* aReflowInput) const {
-#ifdef DEBUG
-  nsPoint offset =
-#endif
-      AdjustReflowInputForScrollbars(aReflowInput);
-
-  NS_ASSERTION(GetAbsoluteContainingBlock()->GetChildList().IsEmpty() ||
-                   (offset.x == 0 && offset.y == 0),
-               "We don't handle correct positioning of fixed frames with "
-               "scrollbars in odd positions");
-
-  nsRect rect(0, 0, aReflowInput->ComputedWidth(),
-              aReflowInput->ComputedHeight());
-
+  const nsPoint origin = AdjustReflowInputForScrollbars(aReflowInput);
+  nsRect rect(origin, aReflowInput->ComputedPhysicalSize());
   rect.SizeTo(AdjustViewportSizeForFixedPosition(rect));
 
   return rect;
@@ -348,7 +329,6 @@ void ViewportFrame::Reflow(nsPresContext* aPresContext,
                            nsReflowStatus& aStatus) {
   MarkInReflow();
   DO_GLOBAL_REFLOW_COUNT("ViewportFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aDesiredSize, aStatus);
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
   NS_FRAME_TRACE_REFLOW_IN("ViewportFrame::Reflow");
 
@@ -359,7 +339,7 @@ void ViewportFrame::Reflow(nsPresContext* aPresContext,
   // Set our size up front, since some parts of reflow depend on it
   // being already set.  Note that the computed height may be
   // unconstrained; that's ok.  Consumers should watch out for that.
-  SetSize(nsSize(aReflowInput.ComputedWidth(), aReflowInput.ComputedHeight()));
+  SetSize(aReflowInput.ComputedPhysicalSize());
 
   // Reflow the main content first so that the placeholders of the
   // fixed-position frames will be in the right places on an initial
@@ -382,6 +362,12 @@ void ViewportFrame::Reflow(nsPresContext* aPresContext,
 
       // Reflow the frame
       kidReflowInput.SetComputedBSize(aReflowInput.ComputedBSize());
+      if (aReflowInput.IsBResizeForWM(kidWM)) {
+        kidReflowInput.SetBResize(true);
+      }
+      if (aReflowInput.IsBResizeForPercentagesForWM(kidWM)) {
+        kidReflowInput.SetBResizeForPercentages(true);
+      }
       ReflowChild(kidFrame, aPresContext, kidDesiredSize, kidReflowInput, 0, 0,
                   ReflowChildFlags::Default, aStatus);
       kidBSize = kidDesiredSize.BSize(wm);
