@@ -41,7 +41,7 @@ eMathMLFrameType nsMathMLTokenFrame::GetMathMLFrameType() {
 
   StyleMathVariant mathVariant = StyleFont()->mMathVariant;
   if ((mathVariant == StyleMathVariant::None &&
-       (StyleFont()->mFont.style == FontSlantStyle::Italic() ||
+       (StyleFont()->mFont.style.IsItalic() ||
         HasAnyStateBits(NS_FRAME_IS_IN_SINGLE_CHAR_MI))) ||
       mathVariant == StyleMathVariant::Italic ||
       mathVariant == StyleMathVariant::BoldItalic ||
@@ -89,23 +89,23 @@ void nsMathMLTokenFrame::MarkTextFramesAsTokenMathML() {
 }
 
 void nsMathMLTokenFrame::SetInitialChildList(ChildListID aListID,
-                                             nsFrameList& aChildList) {
+                                             nsFrameList&& aChildList) {
   // First, let the base class do its work
-  nsMathMLContainerFrame::SetInitialChildList(aListID, aChildList);
+  nsMathMLContainerFrame::SetInitialChildList(aListID, std::move(aChildList));
   MarkTextFramesAsTokenMathML();
 }
 
 void nsMathMLTokenFrame::AppendFrames(ChildListID aListID,
-                                      nsFrameList& aChildList) {
-  nsMathMLContainerFrame::AppendFrames(aListID, aChildList);
+                                      nsFrameList&& aChildList) {
+  nsMathMLContainerFrame::AppendFrames(aListID, std::move(aChildList));
   MarkTextFramesAsTokenMathML();
 }
 
 void nsMathMLTokenFrame::InsertFrames(
     ChildListID aListID, nsIFrame* aPrevFrame,
-    const nsLineList::iterator* aPrevFrameLine, nsFrameList& aChildList) {
+    const nsLineList::iterator* aPrevFrameLine, nsFrameList&& aChildList) {
   nsMathMLContainerFrame::InsertFrames(aListID, aPrevFrame, aPrevFrameLine,
-                                       aChildList);
+                                       std::move(aChildList));
   MarkTextFramesAsTokenMathML();
 }
 
@@ -116,8 +116,6 @@ void nsMathMLTokenFrame::Reflow(nsPresContext* aPresContext,
   MarkInReflow();
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
 
-  mPresentationData.flags &= ~NS_MATHML_ERROR;
-
   // initializations needed for empty markup like <mtag></mtag>
   aDesiredSize.ClearSize();
   aDesiredSize.SetBlockStartAscent(0);
@@ -145,22 +143,28 @@ void nsMathMLTokenFrame::Reflow(nsPresContext* aPresContext,
   FinalizeReflow(aReflowInput.mRenderingContext->GetDrawTarget(), aDesiredSize);
 
   aStatus.Reset();  // This type of frame can't be split.
-  NS_FRAME_SET_TRUNCATION(aStatus, aReflowInput, aDesiredSize);
 }
 
 // For token elements, mBoundingMetrics is computed at the ReflowToken
 // pass, it is not computed here because our children may be text frames
 // that do not implement the GetBoundingMetrics() interface.
 /* virtual */
-nsresult nsMathMLTokenFrame::Place(DrawTarget* aDrawTarget, bool aPlaceOrigin,
+nsresult nsMathMLTokenFrame::Place(DrawTarget* aDrawTarget,
+                                   const PlaceFlags& aFlags,
                                    ReflowOutput& aDesiredSize) {
   mBoundingMetrics = nsBoundingMetrics();
   for (nsIFrame* childFrame : PrincipalChildList()) {
     ReflowOutput childSize(aDesiredSize.GetWritingMode());
-    GetReflowAndBoundingMetricsFor(childFrame, childSize,
-                                   childSize.mBoundingMetrics, nullptr);
+    nsBoundingMetrics bmChild;
+    GetReflowAndBoundingMetricsFor(childFrame, childSize, bmChild, nullptr);
+    auto childMargin = GetMarginForPlace(aFlags, childFrame);
+    bmChild.ascent += childMargin.top;
+    bmChild.descent += childMargin.bottom;
+    bmChild.rightBearing += childMargin.LeftRight();
+    bmChild.width += childMargin.LeftRight();
+
     // compute and cache the bounding metrics
-    mBoundingMetrics += childSize.mBoundingMetrics;
+    mBoundingMetrics += bmChild;
   }
 
   RefPtr<nsFontMetrics> fm =
@@ -174,17 +178,30 @@ nsresult nsMathMLTokenFrame::Place(DrawTarget* aDrawTarget, bool aPlaceOrigin,
   aDesiredSize.Height() = aDesiredSize.BlockStartAscent() +
                           std::max(mBoundingMetrics.descent, descent);
 
-  if (aPlaceOrigin) {
-    nscoord dy, dx = 0;
+  // Apply width/height to math content box.
+  auto sizes = GetWidthAndHeightForPlaceAdjustment(aFlags);
+  auto shiftX = ApplyAdjustmentForWidthAndHeight(aFlags, sizes, aDesiredSize,
+                                                 mBoundingMetrics);
+
+  // Add padding+border.
+  auto borderPadding = GetBorderPaddingForPlace(aFlags);
+  InflateReflowAndBoundingMetrics(borderPadding, aDesiredSize,
+                                  mBoundingMetrics);
+
+  if (!aFlags.contains(PlaceFlag::MeasureOnly)) {
+    nscoord dx = borderPadding.left;
+    dx += shiftX;
     for (nsIFrame* childFrame : PrincipalChildList()) {
       ReflowOutput childSize(aDesiredSize.GetWritingMode());
       GetReflowAndBoundingMetricsFor(childFrame, childSize,
                                      childSize.mBoundingMetrics);
+      auto childMargin = GetMarginForPlace(aFlags, childFrame);
 
       // place and size the child; (dx,0) makes the caret happy - bug 188146
-      dy = childSize.Height() == 0
-               ? 0
-               : aDesiredSize.BlockStartAscent() - childSize.BlockStartAscent();
+      nscoord dy = childSize.Height() == 0
+                       ? 0
+                       : aDesiredSize.BlockStartAscent() -
+                             childSize.BlockStartAscent() + childMargin.top;
       FinishReflowChild(childFrame, PresContext(), childSize, nullptr, dx, dy,
                         ReflowChildFlags::Default);
       dx += childSize.Width();
