@@ -9,17 +9,11 @@
 #include <algorithm>
 #include "gfxFontUtils.h"
 #include "gfxPlatformFontList.h"
-#include "mozilla/dom/CSSFontFaceRule.h"
 #include "mozilla/dom/FontFaceBinding.h"
 #include "mozilla/dom/FontFaceSetImpl.h"
-#include "mozilla/dom/TypedArray.h"
-#include "mozilla/dom/UnionTypes.h"
-#include "mozilla/ServoBindings.h"
 #include "mozilla/ServoCSSParser.h"
-#include "mozilla/ServoUtils.h"
 #include "mozilla/StaticPrefs_layout.h"
 #include "mozilla/dom/Document.h"
-#include "nsStyleUtil.h"
 
 namespace mozilla {
 namespace dom {
@@ -62,9 +56,7 @@ FontFaceImpl::FontFaceImpl(FontFace* aOwner, FontFaceSetImpl* aFontFaceSet)
     : mOwner(aOwner),
       mStatus(FontFaceLoadStatus::Unloaded),
       mSourceType(SourceType(0)),
-      mFontFaceSet(aFontFaceSet),
-      mUnicodeRangeDirty(true),
-      mInFontFaceSet(false) {}
+      mFontFaceSet(aFontFaceSet) {}
 
 FontFaceImpl::~FontFaceImpl() {
   // Assert that we don't drop any FontFace objects during a Servo traversal,
@@ -80,9 +72,18 @@ void FontFaceImpl::AssertIsOnOwningThread() const {
 }
 #endif
 
+void FontFaceImpl::StopKeepingOwnerAlive() {
+  if (mKeepingOwnerAlive) {
+    mKeepingOwnerAlive = false;
+    MOZ_ASSERT(mOwner);
+    mOwner->Release();
+  }
+}
+
 void FontFaceImpl::Destroy() {
   mInFontFaceSet = false;
   SetUserFontEntry(nullptr);
+  StopKeepingOwnerAlive();
   mOwner = nullptr;
 }
 
@@ -106,7 +107,7 @@ static FontFaceLoadStatus LoadStateToStatus(
 already_AddRefed<FontFaceImpl> FontFaceImpl::CreateForRule(
     FontFace* aOwner, FontFaceSetImpl* aFontFaceSet,
     StyleLockedFontFaceRule* aRule) {
-  RefPtr<FontFaceImpl> obj = new FontFaceImpl(aOwner, aFontFaceSet);
+  auto obj = MakeRefPtr<FontFaceImpl>(aOwner, aFontFaceSet);
   obj->mRule = aRule;
   obj->mSourceType = eSourceType_FontFaceRule;
   obj->mInFontFaceSet = true;
@@ -351,14 +352,6 @@ void FontFaceImpl::DoLoad() {
   if (!CreateUserFontEntry()) {
     return;
   }
-
-  if (!NS_IsMainThread()) {
-    NS_DispatchToMainThread(NS_NewRunnableFunction(
-        "FontFaceImpl::DoLoad",
-        [entry = RefPtr{mUserFontEntry}]() { entry->Load(); }));
-    return;
-  }
-
   mUserFontEntry->Load();
 }
 
@@ -400,6 +393,7 @@ void FontFaceImpl::UpdateOwnerPromise() {
   }
 
   if (NS_WARN_IF(!mOwner)) {
+    MOZ_DIAGNOSTIC_ASSERT(!mKeepingOwnerAlive);
     return;
   }
 
@@ -412,6 +406,16 @@ void FontFaceImpl::UpdateOwnerPromise() {
       mOwner->MaybeReject(NS_ERROR_DOM_NETWORK_ERR);
     }
   }
+
+  const bool shouldKeepOwnerAlive = mStatus == FontFaceLoadStatus::Loading;
+  if (shouldKeepOwnerAlive != mKeepingOwnerAlive) {
+    mKeepingOwnerAlive = shouldKeepOwnerAlive;
+    if (shouldKeepOwnerAlive) {
+      mOwner->AddRef();
+    } else {
+      mOwner->Release();
+    }
+  }
 }
 
 // Boolean result indicates whether the value of the descriptor was actually
@@ -480,8 +484,7 @@ bool FontFaceImpl::SetDescriptors(const nsACString& aFamily,
        !setDesc(eCSSFontDesc_FontVariationSettings,
                 aDescriptors.mVariationSettings)) ||
       !setDesc(eCSSFontDesc_Display, aDescriptors.mDisplay) ||
-      (StaticPrefs::layout_css_font_metrics_overrides_enabled() &&
-       (!setDesc(eCSSFontDesc_AscentOverride, aDescriptors.mAscentOverride) ||
+      ((!setDesc(eCSSFontDesc_AscentOverride, aDescriptors.mAscentOverride) ||
         !setDesc(eCSSFontDesc_DescentOverride, aDescriptors.mDescentOverride) ||
         !setDesc(eCSSFontDesc_LineGapOverride,
                  aDescriptors.mLineGapOverride))) ||
@@ -592,9 +595,6 @@ bool FontFaceImpl::GetAttributes(gfxUserFontAttributes& aAttr) {
   if (Servo_FontFaceRule_GetFontStyle(data, &styleDesc)) {
     aAttr.mRangeFlags &= ~gfxFontEntry::RangeFlags::eAutoSlantStyle;
     switch (styleDesc.tag) {
-      case StyleComputedFontStyleDescriptor::Tag::Normal:
-        aAttr.mStyle = SlantStyleRange(FontSlantStyle::NORMAL);
-        break;
       case StyleComputedFontStyleDescriptor::Tag::Italic:
         aAttr.mStyle = SlantStyleRange(FontSlantStyle::ITALIC);
         break;
@@ -684,25 +684,40 @@ bool FontFaceImpl::IsInFontFaceSet(FontFaceSetImpl* aFontFaceSet) const {
 void FontFaceImpl::AddFontFaceSet(FontFaceSetImpl* aFontFaceSet) {
   MOZ_ASSERT(!IsInFontFaceSet(aFontFaceSet));
 
-  if (mFontFaceSet == aFontFaceSet) {
-    mInFontFaceSet = true;
+  auto doAddFontFaceSet = [&]() {
+    if (mFontFaceSet == aFontFaceSet) {
+      mInFontFaceSet = true;
+    } else {
+      mOtherFontFaceSets.AppendElement(aFontFaceSet);
+    }
+  };
+
+  if (mUserFontEntry) {
+    AutoWriteLock lock(mUserFontEntry->Lock());
+    doAddFontFaceSet();
   } else {
-    mOtherFontFaceSets.AppendElement(aFontFaceSet);
+    doAddFontFaceSet();
   }
 }
 
 void FontFaceImpl::RemoveFontFaceSet(FontFaceSetImpl* aFontFaceSet) {
   MOZ_ASSERT(IsInFontFaceSet(aFontFaceSet));
 
-  if (mFontFaceSet == aFontFaceSet) {
-    mInFontFaceSet = false;
-  } else {
-    mOtherFontFaceSets.RemoveElement(aFontFaceSet);
-  }
+  auto doRemoveFontFaceSet = [&]() {
+    if (mFontFaceSet == aFontFaceSet) {
+      mInFontFaceSet = false;
+    } else {
+      mOtherFontFaceSets.RemoveElement(aFontFaceSet);
+    }
+  };
 
-  // The caller should be holding a strong reference to the FontFaceSetImpl.
   if (mUserFontEntry) {
-    mUserFontEntry->CheckUserFontSet();
+    AutoWriteLock lock(mUserFontEntry->Lock());
+    doRemoveFontFaceSet();
+    // The caller should be holding a strong reference to the FontFaceSetImpl.
+    mUserFontEntry->CheckUserFontSetLocked();
+  } else {
+    doRemoveFontFaceSet();
   }
 }
 
@@ -717,7 +732,7 @@ gfxCharacterMap* FontFaceImpl::GetUnicodeRangeAsCharacterMap() {
 
   Span<const StyleUnicodeRange> ranges(rangesPtr, len);
   if (!ranges.IsEmpty()) {
-    RefPtr<gfxCharacterMap> charMap = new gfxCharacterMap();
+    auto charMap = MakeRefPtr<gfxCharacterMap>();
     for (auto& range : ranges) {
       charMap->SetRange(range.start, range.end);
     }
@@ -745,7 +760,7 @@ void FontFaceImpl::Entry::SetLoadState(UserFontLoadState aLoadState) {
 
   nsTArray<RefPtr<FontFaceImpl>> fontFaces;
   {
-    MutexAutoLock lock(mMutex);
+    AutoReadLock lock(mLock);
     fontFaces.SetCapacity(mFontFaces.Length());
     for (FontFaceImpl* f : mFontFaces) {
       fontFaces.AppendElement(f);
@@ -767,7 +782,7 @@ void FontFaceImpl::Entry::SetLoadState(UserFontLoadState aLoadState) {
 /* virtual */
 void FontFaceImpl::Entry::GetUserFontSets(
     nsTArray<RefPtr<gfxUserFontSet>>& aResult) {
-  MutexAutoLock lock(mMutex);
+  AutoReadLock lock(mLock);
 
   aResult.Clear();
 
@@ -792,7 +807,7 @@ void FontFaceImpl::Entry::GetUserFontSets(
 
 /* virtual */ already_AddRefed<gfxUserFontSet>
 FontFaceImpl::Entry::GetUserFontSet() const {
-  MutexAutoLock lock(mMutex);
+  AutoReadLock lock(mLock);
   if (mFontSet) {
     return do_AddRef(mFontSet);
   }
@@ -825,7 +840,7 @@ void FontFaceImpl::Entry::CheckUserFontSetLocked() {
 }
 
 void FontFaceImpl::Entry::FindFontFaceOwners(nsTHashSet<FontFace*>& aOwners) {
-  MutexAutoLock lock(mMutex);
+  AutoReadLock lock(mLock);
   for (FontFaceImpl* f : mFontFaces) {
     if (FontFace* owner = f->GetOwner()) {
       aOwners.Insert(owner);
@@ -834,13 +849,13 @@ void FontFaceImpl::Entry::FindFontFaceOwners(nsTHashSet<FontFace*>& aOwners) {
 }
 
 void FontFaceImpl::Entry::AddFontFace(FontFaceImpl* aFontFace) {
-  MutexAutoLock lock(mMutex);
+  AutoWriteLock lock(mLock);
   mFontFaces.AppendElement(aFontFace);
   CheckUserFontSetLocked();
 }
 
 void FontFaceImpl::Entry::RemoveFontFace(FontFaceImpl* aFontFace) {
-  MutexAutoLock lock(mMutex);
+  AutoWriteLock lock(mLock);
   mFontFaces.RemoveElement(aFontFace);
   CheckUserFontSetLocked();
 }
