@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-// vim:cindent:ts=4:et:sw=4:
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+// vim:cindent:ts=2:et:sw=2:
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -24,35 +24,32 @@
 using namespace mozilla;
 using namespace mozilla::layout;
 
-namespace css = mozilla::css;
-
 #undef DEBUG_TABLE_STRATEGY
 
-BasicTableLayoutStrategy::BasicTableLayoutStrategy(nsTableFrame *aTableFrame)
+BasicTableLayoutStrategy::BasicTableLayoutStrategy(nsTableFrame* aTableFrame)
     : nsITableLayoutStrategy(nsITableLayoutStrategy::Auto),
       mTableFrame(aTableFrame) {
   MarkIntrinsicISizesDirty();
 }
 
 /* virtual */
-BasicTableLayoutStrategy::~BasicTableLayoutStrategy() {}
+BasicTableLayoutStrategy::~BasicTableLayoutStrategy() = default;
 
-/* virtual */ nscoord BasicTableLayoutStrategy::GetMinISize(
-    gfxContext *aRenderingContext) {
-  DISPLAY_MIN_WIDTH(mTableFrame, mMinISize);
-  if (mMinISize == NS_INTRINSIC_WIDTH_UNKNOWN) {
+/* virtual */
+nscoord BasicTableLayoutStrategy::GetMinISize(gfxContext* aRenderingContext) {
+  if (mMinISize == NS_INTRINSIC_ISIZE_UNKNOWN) {
     ComputeIntrinsicISizes(aRenderingContext);
   }
   return mMinISize;
 }
 
-/* virtual */ nscoord BasicTableLayoutStrategy::GetPrefISize(
-    gfxContext *aRenderingContext, bool aComputingSize) {
-  DISPLAY_PREF_WIDTH(mTableFrame, mPrefISize);
-  NS_ASSERTION((mPrefISize == NS_INTRINSIC_WIDTH_UNKNOWN) ==
-                   (mPrefISizePctExpand == NS_INTRINSIC_WIDTH_UNKNOWN),
+/* virtual */
+nscoord BasicTableLayoutStrategy::GetPrefISize(gfxContext* aRenderingContext,
+                                               bool aComputingSize) {
+  NS_ASSERTION((mPrefISize == NS_INTRINSIC_ISIZE_UNKNOWN) ==
+                   (mPrefISizePctExpand == NS_INTRINSIC_ISIZE_UNKNOWN),
                "dirtyness out of sync");
-  if (mPrefISize == NS_INTRINSIC_WIDTH_UNKNOWN) {
+  if (mPrefISize == NS_INTRINSIC_ISIZE_UNKNOWN) {
     ComputeIntrinsicISizes(aRenderingContext);
   }
   return aComputingSize ? mPrefISizePctExpand : mPrefISize;
@@ -72,13 +69,16 @@ struct CellISizeInfo {
   float prefPercent;
 };
 
-// Used for both column and cell calculations.  The parts needed only
-// for cells are skipped when aIsCell is false.
-static CellISizeInfo GetISizeInfo(gfxContext *aRenderingContext,
-                                  nsIFrame *aFrame, WritingMode aWM,
+// A helper for ComputeColumnIntrinsicISizes(), used for both column and cell
+// intrinsic inline size calculations. The parts needed only for cells are
+// skipped when aIsCell is false.
+static CellISizeInfo GetISizeInfo(gfxContext* aRenderingContext,
+                                  nsIFrame* aFrame, WritingMode aWM,
                                   bool aIsCell) {
+  MOZ_ASSERT(aFrame->GetWritingMode() == aWM,
+             "The caller is expected to pass aFrame's writing mode!");
   nscoord minCoord, prefCoord;
-  const nsStylePosition *stylePos = aFrame->StylePosition();
+  const nsStylePosition* stylePos = aFrame->StylePosition();
   bool isQuirks =
       aFrame->PresContext()->CompatibilityMode() == eCompatibility_NavQuirks;
   nscoord boxSizingToBorderEdge = 0;
@@ -87,8 +87,30 @@ static CellISizeInfo GetISizeInfo(gfxContext *aRenderingContext,
     // wrapping inside of it should not apply font size inflation.
     AutoMaybeDisableFontInflation an(aFrame);
 
-    minCoord = aFrame->GetMinISize(aRenderingContext);
-    prefCoord = aFrame->GetPrefISize(aRenderingContext);
+    // Resolve the cell's block size 'cellBSize' as a percentage basis, in case
+    // it impacts its children's inline-size contributions (e.g. via percentage
+    // block size + aspect-ratio). However, this behavior might not be
+    // web-compatible (Bug 1461852).
+    //
+    // Note that if the cell *itself* has a percentage-based block size, we
+    // treat it as unresolvable here by using an unconstrained cbBSize. It will
+    // be resolved during the "special bsize reflow" pass if the table has a
+    // specified block size. See nsTableFrame::Reflow() and
+    // ReflowInput::Flags::mSpecialBSizeReflow.
+    const nscoord cbBSize = NS_UNCONSTRAINEDSIZE;
+    const nscoord contentEdgeToBoxSizingBSize =
+        stylePos->mBoxSizing == StyleBoxSizing::Border
+            ? aFrame->IntrinsicBSizeOffsets().BorderPadding()
+            : 0;
+    const nscoord cellBSize = nsIFrame::ComputeBSizeValueAsPercentageBasis(
+        stylePos->BSize(aWM), stylePos->MinBSize(aWM), stylePos->MaxBSize(aWM),
+        cbBSize, contentEdgeToBoxSizingBSize);
+
+    const IntrinsicSizeInput input(
+        aRenderingContext, Nothing(),
+        Some(LogicalSize(aWM, NS_UNCONSTRAINEDSIZE, cellBSize)));
+    minCoord = aFrame->GetMinISize(input);
+    prefCoord = aFrame->GetPrefISize(input);
     // Until almost the end of this function, minCoord and prefCoord
     // represent the box-sizing based isize values (which mean they
     // should include inline padding and border width when
@@ -97,22 +119,13 @@ static CellISizeInfo GetISizeInfo(gfxContext *aRenderingContext,
     // outer edges near the end of this function.
 
     // XXX Should we ignore percentage padding?
-    nsIFrame::IntrinsicISizeOffsetData offsets =
-        aFrame->IntrinsicISizeOffsets();
-
-    // In quirks mode, table cell isize should be content-box,
-    // but bsize should be border box.
-    // Because of this historic anomaly, we do not use quirk.css.
-    // (We can't specify one value of box-sizing for isize and another
-    // for bsize).
-    // For this reason, we also do not use box-sizing for just one of
-    // them, as this may be confusing.
-    if (isQuirks || stylePos->mBoxSizing == StyleBoxSizing::Content) {
-      boxSizingToBorderEdge = offsets.hPadding + offsets.hBorder;
+    nsIFrame::IntrinsicSizeOffsetData offsets = aFrame->IntrinsicISizeOffsets();
+    if (stylePos->mBoxSizing == StyleBoxSizing::Content) {
+      boxSizingToBorderEdge = offsets.padding + offsets.border;
     } else {
-      // StyleBoxSizing::Border and standards-mode
-      minCoord += offsets.hPadding + offsets.hBorder;
-      prefCoord += offsets.hPadding + offsets.hBorder;
+      // StyleBoxSizing::Border
+      minCoord += offsets.padding + offsets.border;
+      prefCoord += offsets.padding + offsets.border;
     }
   } else {
     minCoord = 0;
@@ -121,97 +134,101 @@ static CellISizeInfo GetISizeInfo(gfxContext *aRenderingContext,
   float prefPercent = 0.0f;
   bool hasSpecifiedISize = false;
 
-  const nsStyleCoord &iSize = stylePos->ISize(aWM);
-  nsStyleUnit unit = iSize.GetUnit();
-  // NOTE: We're ignoring calc() units with percentages here, for lack of a
-  // sensible idea for what to do with them.  This means calc() with
-  // percentages is basically handled like 'auto' for table cells and
+  const auto& iSize = stylePos->ISize(aWM);
+  // NOTE: We're ignoring calc() units with both lengths and percentages here,
+  // for lack of a sensible idea for what to do with them.  This means calc()
+  // with percentages is basically handled like 'auto' for table cells and
   // columns.
   if (iSize.ConvertsToLength()) {
     hasSpecifiedISize = true;
-    // Note: since ComputeISizeValue was designed to return content-box
-    // isize, it will (in some cases) subtract the box-sizing edges.
-    // We prevent this unwanted behavior by calling it with
-    // aContentEdgeToBoxSizing and aBoxSizingToMarginEdge set to 0.
-    nscoord c = aFrame->ComputeISizeValue(aRenderingContext, 0, 0, 0, iSize);
+    nscoord c = iSize.ToLength();
     // Quirk: A cell with "nowrap" set and a coord value for the
     // isize which is bigger than the intrinsic minimum isize uses
     // that coord value as the minimum isize.
     // This is kept up-to-date with dynamic changes to nowrap by code in
     // nsTableCellFrame::AttributeChanged
     if (aIsCell && c > minCoord && isQuirks &&
-        aFrame->GetContent()->AsElement()->HasAttr(kNameSpaceID_None,
-                                                   nsGkAtoms::nowrap)) {
+        aFrame->GetContent()->AsElement()->HasAttr(nsGkAtoms::nowrap)) {
       minCoord = c;
     }
     prefCoord = std::max(c, minCoord);
-  } else if (unit == eStyleUnit_Percent) {
-    prefPercent = iSize.GetPercentValue();
-  } else if (unit == eStyleUnit_Enumerated && aIsCell) {
-    switch (iSize.GetIntValue()) {
-      case NS_STYLE_WIDTH_MAX_CONTENT:
+  } else if (iSize.ConvertsToPercentage()) {
+    prefPercent = iSize.ToPercentage();
+  } else if (aIsCell) {
+    switch (iSize.tag) {
+      case StyleSize::Tag::MaxContent:
         // 'inline-size' only affects pref isize, not min
         // isize, so don't change anything
         break;
-      case NS_STYLE_WIDTH_MIN_CONTENT:
+      case StyleSize::Tag::MinContent:
         prefCoord = minCoord;
         break;
-      case NS_STYLE_WIDTH_FIT_CONTENT:
-      case NS_STYLE_WIDTH_AVAILABLE:
-        // act just like 'inline-size: auto'
+      case StyleSize::Tag::MozAvailable:
+      case StyleSize::Tag::WebkitFillAvailable:
+      case StyleSize::Tag::Stretch:
+      case StyleSize::Tag::FitContent:
+      case StyleSize::Tag::FitContentFunction:
+        // TODO: Bug 1708310: Make sure fit-content() work properly in table.
+      case StyleSize::Tag::Auto:
+      case StyleSize::Tag::LengthPercentage:
+      case StyleSize::Tag::AnchorSizeFunction:
         break;
-      default:
-        NS_NOTREACHED("unexpected enumerated value");
     }
   }
 
-  nsStyleCoord maxISize(stylePos->MaxISize(aWM));
-  if (maxISize.GetUnit() == eStyleUnit_Enumerated) {
-    if (!aIsCell || maxISize.GetIntValue() == NS_STYLE_WIDTH_AVAILABLE) {
-      maxISize.SetNoneValue();
-    } else if (maxISize.GetIntValue() == NS_STYLE_WIDTH_FIT_CONTENT) {
-      // for 'max-inline-size', '-moz-fit-content' is like
-      // '-moz-max-content'
-      maxISize.SetIntValue(NS_STYLE_WIDTH_MAX_CONTENT, eStyleUnit_Enumerated);
+  StyleMaxSize maxISize = stylePos->MaxISize(aWM);
+  if (nsIFrame::ToExtremumLength(maxISize)) {
+    if (!aIsCell || maxISize.BehavesLikeStretchOnInlineAxis()) {
+      maxISize = StyleMaxSize::None();
+    } else if (maxISize.IsFitContent() || maxISize.IsFitContentFunction()) {
+      // TODO: Bug 1708310: Make sure fit-content() work properly in table.
+      // for 'max-inline-size', '-moz-fit-content' is like 'max-content'
+      maxISize = StyleMaxSize::MaxContent();
     }
   }
-  unit = maxISize.GetUnit();
   // XXX To really implement 'max-inline-size' well, we'd need to store
   // it separately on the columns.
-  if (maxISize.ConvertsToLength() || unit == eStyleUnit_Enumerated) {
-    nscoord c = aFrame->ComputeISizeValue(aRenderingContext, 0, 0, 0, maxISize);
+  const LogicalSize zeroSize(aWM);
+  if (maxISize.ConvertsToLength() || nsIFrame::ToExtremumLength(maxISize)) {
+    nscoord c = aFrame
+                    ->ComputeISizeValue(
+                        aRenderingContext, aWM, zeroSize, zeroSize, 0, maxISize,
+                        stylePos->BSize(aWM), aFrame->GetAspectRatio())
+                    .mISize;
     minCoord = std::min(c, minCoord);
     prefCoord = std::min(c, prefCoord);
-  } else if (unit == eStyleUnit_Percent) {
-    float p = stylePos->MaxISize(aWM).GetPercentValue();
+  } else if (maxISize.ConvertsToPercentage()) {
+    float p = maxISize.ToPercentage();
     if (p < prefPercent) {
       prefPercent = p;
     }
   }
-  // treat calc() with percentages on max-inline-size just like 'none'.
 
-  nsStyleCoord minISize(stylePos->MinISize(aWM));
-  if (minISize.GetUnit() == eStyleUnit_Enumerated) {
-    if (!aIsCell || minISize.GetIntValue() == NS_STYLE_WIDTH_AVAILABLE) {
-      minISize.SetCoordValue(0);
-    } else if (minISize.GetIntValue() == NS_STYLE_WIDTH_FIT_CONTENT) {
-      // for 'min-inline-size', '-moz-fit-content' is like
-      // '-moz-min-content'
-      minISize.SetIntValue(NS_STYLE_WIDTH_MIN_CONTENT, eStyleUnit_Enumerated);
+  StyleSize minISize = stylePos->MinISize(aWM);
+  if (nsIFrame::ToExtremumLength(maxISize)) {
+    if (!aIsCell || minISize.BehavesLikeStretchOnInlineAxis()) {
+      minISize = StyleSize::LengthPercentage(LengthPercentage::Zero());
+    } else if (minISize.IsFitContent() || minISize.IsFitContentFunction()) {
+      // TODO: Bug 1708310: Make sure fit-content() work properly in table.
+      // for 'min-inline-size', '-moz-fit-content' is like 'min-content'
+      minISize = StyleSize::MinContent();
     }
   }
-  unit = minISize.GetUnit();
-  if (minISize.ConvertsToLength() || unit == eStyleUnit_Enumerated) {
-    nscoord c = aFrame->ComputeISizeValue(aRenderingContext, 0, 0, 0, minISize);
+
+  if (minISize.ConvertsToLength() || nsIFrame::ToExtremumLength(minISize)) {
+    nscoord c = aFrame
+                    ->ComputeISizeValue(
+                        aRenderingContext, aWM, zeroSize, zeroSize, 0, minISize,
+                        stylePos->BSize(aWM), aFrame->GetAspectRatio())
+                    .mISize;
     minCoord = std::max(c, minCoord);
     prefCoord = std::max(c, prefCoord);
-  } else if (unit == eStyleUnit_Percent) {
-    float p = stylePos->MinISize(aWM).GetPercentValue();
+  } else if (minISize.ConvertsToPercentage()) {
+    float p = minISize.ToPercentage();
     if (p > prefPercent) {
       prefPercent = p;
     }
   }
-  // treat calc() with percentages on min-inline-size just like '0'.
 
   // XXX Should col frame have border/padding considered?
   if (aIsCell) {
@@ -222,14 +239,14 @@ static CellISizeInfo GetISizeInfo(gfxContext *aRenderingContext,
   return CellISizeInfo(minCoord, prefCoord, prefPercent, hasSpecifiedISize);
 }
 
-static inline CellISizeInfo GetCellISizeInfo(gfxContext *aRenderingContext,
-                                             nsTableCellFrame *aCellFrame,
+static inline CellISizeInfo GetCellISizeInfo(gfxContext* aRenderingContext,
+                                             nsTableCellFrame* aCellFrame,
                                              WritingMode aWM) {
   return GetISizeInfo(aRenderingContext, aCellFrame, aWM, true);
 }
 
-static inline CellISizeInfo GetColISizeInfo(gfxContext *aRenderingContext,
-                                            nsIFrame *aFrame, WritingMode aWM) {
+static inline CellISizeInfo GetColISizeInfo(gfxContext* aRenderingContext,
+                                            nsIFrame* aFrame, WritingMode aWM) {
   return GetISizeInfo(aRenderingContext, aFrame, aWM, false);
 }
 
@@ -240,9 +257,9 @@ static inline CellISizeInfo GetColISizeInfo(gfxContext *aRenderingContext,
  * browsers are).
  */
 void BasicTableLayoutStrategy::ComputeColumnIntrinsicISizes(
-    gfxContext *aRenderingContext) {
-  nsTableFrame *tableFrame = mTableFrame;
-  nsTableCellMap *cellMap = tableFrame->GetCellMap();
+    gfxContext* aRenderingContext) {
+  nsTableFrame* tableFrame = mTableFrame;
+  nsTableCellMap* cellMap = tableFrame->GetCellMap();
   WritingMode wm = tableFrame->GetWritingMode();
 
   mozilla::AutoStackArena arena;
@@ -252,7 +269,7 @@ void BasicTableLayoutStrategy::ComputeColumnIntrinsicISizes(
   // a colspan.
   int32_t col, col_end;
   for (col = 0, col_end = cellMap->GetColCount(); col < col_end; ++col) {
-    nsTableColFrame *colFrame = tableFrame->GetColFrame(col);
+    nsTableColFrame* colFrame = tableFrame->GetColFrame(col);
     if (!colFrame) {
       NS_ERROR("column frames out of sync with cell map");
       continue;
@@ -285,7 +302,7 @@ void BasicTableLayoutStrategy::ComputeColumnIntrinsicISizes(
     // colspans.
     nsCellMapColumnIterator columnIter(cellMap, col);
     int32_t row, colSpan;
-    nsTableCellFrame *cellFrame;
+    nsTableCellFrame* cellFrame;
     while ((cellFrame = columnIter.GetNextFrame(&row, &colSpan))) {
       if (colSpan > 1) {
         spanningCells.AddCell(colSpan, row, col);
@@ -327,7 +344,7 @@ void BasicTableLayoutStrategy::ComputeColumnIntrinsicISizes(
   // Starting with smaller colspans makes it more likely that we
   // satisfy all the constraints given and don't distribute space to
   // columns where we don't need it.
-  SpanningCellSorter::Item *item;
+  SpanningCellSorter::Item* item;
   int32_t colSpan;
   while ((item = spanningCells.GetNext(&colSpan))) {
     NS_ASSERTION(colSpan > 1,
@@ -335,11 +352,11 @@ void BasicTableLayoutStrategy::ComputeColumnIntrinsicISizes(
     do {
       int32_t row = item->row;
       col = item->col;
-      CellData *cellData = cellMap->GetDataAt(row, col);
+      CellData* cellData = cellMap->GetDataAt(row, col);
       NS_ASSERTION(cellData && cellData->IsOrig(),
                    "bogus result from spanning cell sorter");
 
-      nsTableCellFrame *cellFrame = cellData->GetCellFrame();
+      nsTableCellFrame* cellFrame = cellData->GetCellFrame();
       NS_ASSERTION(cellFrame, "bogus result from spanning cell sorter");
 
       CellISizeInfo info = GetCellISizeInfo(aRenderingContext, cellFrame, wm);
@@ -347,9 +364,10 @@ void BasicTableLayoutStrategy::ComputeColumnIntrinsicISizes(
       if (info.prefPercent > 0.0f) {
         DistributePctISizeToColumns(info.prefPercent, col, colSpan);
       }
-      DistributeISizeToColumns(info.minCoord, col, colSpan, BTLS_MIN_ISIZE,
-                               info.hasSpecifiedISize);
-      DistributeISizeToColumns(info.prefCoord, col, colSpan, BTLS_PREF_ISIZE,
+      DistributeISizeToColumns(info.minCoord, col, colSpan,
+                               BtlsISizeType::MinISize, info.hasSpecifiedISize);
+      DistributeISizeToColumns(info.prefCoord, col, colSpan,
+                               BtlsISizeType::PrefISize,
                                info.hasSpecifiedISize);
     } while ((item = item->next));
 
@@ -357,7 +375,7 @@ void BasicTableLayoutStrategy::ComputeColumnIntrinsicISizes(
     // for each increment of colspan.
 
     for (col = 0, col_end = cellMap->GetColCount(); col < col_end; ++col) {
-      nsTableColFrame *colFrame = tableFrame->GetColFrame(col);
+      nsTableColFrame* colFrame = tableFrame->GetColFrame(col);
       if (!colFrame) {
         NS_ERROR("column frames out of sync with cell map");
         continue;
@@ -382,7 +400,7 @@ void BasicTableLayoutStrategy::ComputeColumnIntrinsicISizes(
   // 100%).  This means layout depends on the order of columns.
   float pct_used = 0.0f;
   for (col = 0, col_end = cellMap->GetColCount(); col < col_end; ++col) {
-    nsTableColFrame *colFrame = tableFrame->GetColFrame(col);
+    nsTableColFrame* colFrame = tableFrame->GetColFrame(col);
     if (!colFrame) {
       NS_ERROR("column frames out of sync with cell map");
       continue;
@@ -398,10 +416,10 @@ void BasicTableLayoutStrategy::ComputeColumnIntrinsicISizes(
 }
 
 void BasicTableLayoutStrategy::ComputeIntrinsicISizes(
-    gfxContext *aRenderingContext) {
+    gfxContext* aRenderingContext) {
   ComputeColumnIntrinsicISizes(aRenderingContext);
 
-  nsTableCellMap *cellMap = mTableFrame->GetCellMap();
+  nsTableCellMap* cellMap = mTableFrame->GetCellMap();
   nscoord min = 0, pref = 0, max_small_pct_pref = 0, nonpct_pref_total = 0;
   float pct_total = 0.0f;  // always from 0.0f - 1.0f
   int32_t colCount = cellMap->GetColCount();
@@ -410,7 +428,7 @@ void BasicTableLayoutStrategy::ComputeIntrinsicISizes(
   nscoord add = mTableFrame->GetColSpacing(colCount);
 
   for (int32_t col = 0; col < colCount; ++col) {
-    nsTableColFrame *colFrame = mTableFrame->GetColFrame(col);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(col);
     if (!colFrame) {
       NS_ERROR("column frames out of sync with cell map");
       continue;
@@ -463,7 +481,9 @@ void BasicTableLayoutStrategy::ComputeIntrinsicISizes(
         (nonpct_pref_total == nscoord_MAX
              ? nscoord_MAX
              : nscoord(float(nonpct_pref_total) / (1.0f - pct_total)));
-    if (large_pct_pref > pref_pct_expand) pref_pct_expand = large_pct_pref;
+    if (large_pct_pref > pref_pct_expand) {
+      pref_pct_expand = large_pct_pref;
+    }
   }
 
   // border-spacing isn't part of the basis for percentages
@@ -478,15 +498,17 @@ void BasicTableLayoutStrategy::ComputeIntrinsicISizes(
   mPrefISizePctExpand = pref_pct_expand;
 }
 
-/* virtual */ void BasicTableLayoutStrategy::MarkIntrinsicISizesDirty() {
-  mMinISize = NS_INTRINSIC_WIDTH_UNKNOWN;
-  mPrefISize = NS_INTRINSIC_WIDTH_UNKNOWN;
-  mPrefISizePctExpand = NS_INTRINSIC_WIDTH_UNKNOWN;
+/* virtual */
+void BasicTableLayoutStrategy::MarkIntrinsicISizesDirty() {
+  mMinISize = NS_INTRINSIC_ISIZE_UNKNOWN;
+  mPrefISize = NS_INTRINSIC_ISIZE_UNKNOWN;
+  mPrefISizePctExpand = NS_INTRINSIC_ISIZE_UNKNOWN;
   mLastCalcISize = nscoord_MIN;
 }
 
-/* virtual */ void BasicTableLayoutStrategy::ComputeColumnISizes(
-    const ReflowInput &aReflowInput) {
+/* virtual */
+void BasicTableLayoutStrategy::ComputeColumnISizes(
+    const ReflowInput& aReflowInput) {
   nscoord iSize = aReflowInput.ComputedISize();
 
   if (mLastCalcISize == iSize) {
@@ -494,22 +516,25 @@ void BasicTableLayoutStrategy::ComputeIntrinsicISizes(
   }
   mLastCalcISize = iSize;
 
-  NS_ASSERTION((mMinISize == NS_INTRINSIC_WIDTH_UNKNOWN) ==
-                   (mPrefISize == NS_INTRINSIC_WIDTH_UNKNOWN),
+  NS_ASSERTION((mMinISize == NS_INTRINSIC_ISIZE_UNKNOWN) ==
+                   (mPrefISize == NS_INTRINSIC_ISIZE_UNKNOWN),
                "dirtyness out of sync");
-  NS_ASSERTION((mMinISize == NS_INTRINSIC_WIDTH_UNKNOWN) ==
-                   (mPrefISizePctExpand == NS_INTRINSIC_WIDTH_UNKNOWN),
+  NS_ASSERTION((mMinISize == NS_INTRINSIC_ISIZE_UNKNOWN) ==
+                   (mPrefISizePctExpand == NS_INTRINSIC_ISIZE_UNKNOWN),
                "dirtyness out of sync");
   // XXX Is this needed?
-  if (mMinISize == NS_INTRINSIC_WIDTH_UNKNOWN) {
+  if (mMinISize == NS_INTRINSIC_ISIZE_UNKNOWN) {
     ComputeIntrinsicISizes(aReflowInput.mRenderingContext);
   }
 
-  nsTableCellMap *cellMap = mTableFrame->GetCellMap();
+  nsTableCellMap* cellMap = mTableFrame->GetCellMap();
   int32_t colCount = cellMap->GetColCount();
-  if (colCount <= 0) return;  // nothing to do
+  if (colCount <= 0) {
+    return;  // nothing to do
+  }
 
-  DistributeISizeToColumns(iSize, 0, colCount, BTLS_FINAL_ISIZE, false);
+  DistributeISizeToColumns(iSize, 0, colCount, BtlsISizeType::FinalISize,
+                           false);
 
 #ifdef DEBUG_TABLE_STRATEGY
   printf("ComputeColumnISizes final\n");
@@ -526,10 +551,10 @@ void BasicTableLayoutStrategy::DistributePctISizeToColumns(float aSpanPrefPct,
   // and to reduce aSpanPrefPct by columns that already have % isize
 
   int32_t scol, scol_end;
-  nsTableCellMap *cellMap = mTableFrame->GetCellMap();
+  nsTableCellMap* cellMap = mTableFrame->GetCellMap();
   for (scol = aFirstCol, scol_end = aFirstCol + aColCount; scol < scol_end;
        ++scol) {
-    nsTableColFrame *scolFrame = mTableFrame->GetColFrame(scol);
+    nsTableColFrame* scolFrame = mTableFrame->GetColFrame(scol);
     if (!scolFrame) {
       NS_ERROR("column frames out of sync with cell map");
       continue;
@@ -556,7 +581,7 @@ void BasicTableLayoutStrategy::DistributePctISizeToColumns(float aSpanPrefPct,
   const bool spanHasNonPctPref = nonPctTotalPrefISize > 0;  // Loop invariant
   for (scol = aFirstCol, scol_end = aFirstCol + aColCount; scol < scol_end;
        ++scol) {
-    nsTableColFrame *scolFrame = mTableFrame->GetColFrame(scol);
+    nsTableColFrame* scolFrame = mTableFrame->GetColFrame(scol);
     if (!scolFrame) {
       NS_ERROR("column frames out of sync with cell map");
       continue;
@@ -607,7 +632,7 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
     nscoord aISize, int32_t aFirstCol, int32_t aColCount,
     BtlsISizeType aISizeType, bool aSpanHasSpecifiedISize) {
   NS_ASSERTION(
-      aISizeType != BTLS_FINAL_ISIZE ||
+      aISizeType != BtlsISizeType::FinalISize ||
           (aFirstCol == 0 &&
            aColCount == mTableFrame->GetCellMap()->GetColCount()),
       "Computing final column isizes, but didn't get full column range");
@@ -622,7 +647,7 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
       subtract += mTableFrame->GetColSpacing(col - 1);
     }
   }
-  if (aISizeType == BTLS_FINAL_ISIZE) {
+  if (aISizeType == BtlsISizeType::FinalISize) {
     // If we're computing final col-isize, then aISize initially includes
     // border spacing on the table's far istart + far iend edge, too.  Need
     // to subtract those out, too.
@@ -691,9 +716,9 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
   int32_t numNonSpecZeroISizeCols = 0;
 
   int32_t col;
-  nsTableCellMap *cellMap = mTableFrame->GetCellMap();
+  nsTableCellMap* cellMap = mTableFrame->GetCellMap();
   for (col = aFirstCol; col < aFirstCol + aColCount; ++col) {
-    nsTableColFrame *colFrame = mTableFrame->GetColFrame(col);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(col);
     if (!colFrame) {
       NS_ERROR("column frames out of sync with cell map");
       continue;
@@ -707,7 +732,7 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
       if (val < min_iSize) {
         val = min_iSize;
       }
-      guess_min_pct += val;
+      guess_min_pct = NSCoordSaturatingAdd(guess_min_pct, val);
       guess_pref = NSCoordSaturatingAdd(guess_pref, val);
     } else {
       nscoord pref_iSize = colFrame->GetPrefCoord();
@@ -715,7 +740,7 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
         ++numInfiniteISizeCols;
       }
       guess_pref = NSCoordSaturatingAdd(guess_pref, pref_iSize);
-      guess_min_pct += min_iSize;
+      guess_min_pct = NSCoordSaturatingAdd(guess_min_pct, min_iSize);
       if (colFrame->GetHasSpecifiedCoord()) {
         // we'll add on the rest of guess_min_spec outside the
         // loop
@@ -755,13 +780,14 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
     float f;
   } basis;  // the sum of the statistic over columns to divide it
   if (aISize < guess_pref) {
-    if (aISizeType != BTLS_FINAL_ISIZE && aISize <= guess_min) {
+    if (aISizeType != BtlsISizeType::FinalISize && aISize <= guess_min) {
       // Return early -- we don't have any extra space to distribute.
       return;
     }
-    NS_ASSERTION(!(aISizeType == BTLS_FINAL_ISIZE && aISize < guess_min),
-                 "Table inline-size is less than the "
-                 "sum of its columns' min inline-sizes");
+    NS_ASSERTION(
+        !(aISizeType == BtlsISizeType::FinalISize && aISize < guess_min),
+        "Table inline-size is less than the sum of its columns' min "
+        "inline-sizes");
     if (aISize < guess_min_pct) {
       l2t = FLEX_PCT_SMALL;
       space = aISize - guess_min;
@@ -807,7 +833,7 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
 #endif
 
   for (col = aFirstCol; col < aFirstCol + aColCount; ++col) {
-    nsTableColFrame *colFrame = mTableFrame->GetColFrame(col);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(col);
     if (!colFrame) {
       NS_ERROR("column frames out of sync with cell map");
       continue;
@@ -835,7 +861,8 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
           if (pct_minus_min > 0) {
             float c = float(space) / float(basis.c);
             basis.c -= pct_minus_min;
-            col_iSize += NSToCoordRound(float(pct_minus_min) * c);
+            col_iSize = NSCoordSaturatingAdd(
+                col_iSize, NSToCoordRound(float(pct_minus_min) * c));
           }
         }
         break;
@@ -849,11 +876,14 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
             col_iSize = col_iSize_before_adjust = col_min;
             if (pref_minus_min != 0) {
               float c = float(space) / float(basis.c);
-              basis.c -= pref_minus_min;
-              col_iSize += NSToCoordRound(float(pref_minus_min) * c);
+              basis.c = NSCoordSaturatingSubtract(basis.c, pref_minus_min,
+                                                  nscoord_MAX);
+              col_iSize = NSCoordSaturatingAdd(
+                  col_iSize, NSToCoordRound(float(pref_minus_min) * c));
             }
-          } else
+          } else {
             col_iSize = col_iSize_before_adjust = colFrame->GetMinCoord();
+          }
         }
         break;
       case FLEX_FLEX_SMALL:
@@ -884,7 +914,8 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
             }
             basis.c =
                 NSCoordSaturatingSubtract(basis.c, pref_minus_min, nscoord_MAX);
-            col_iSize += NSToCoordRound(float(pref_minus_min) * c);
+            col_iSize = NSCoordSaturatingAdd(
+                col_iSize, NSToCoordRound(float(pref_minus_min) * c));
           }
         }
         break;
@@ -898,8 +929,10 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
               col_iSize = nscoord_MAX;
             } else {
               float c = float(space) / float(basis.c);
-              basis.c -= col_iSize;
-              col_iSize += NSToCoordRound(float(col_iSize) * c);
+              basis.c =
+                  NSCoordSaturatingSubtract(basis.c, col_iSize, nscoord_MAX);
+              col_iSize = NSCoordSaturatingAdd(
+                  col_iSize, NSToCoordRound(float(col_iSize) * c));
             }
           }
         }
@@ -925,8 +958,10 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
               "wrong case");
           if (col_iSize != 0) {
             float c = float(space) / float(basis.c);
-            basis.c -= col_iSize;
-            col_iSize += NSToCoordRound(float(col_iSize) * c);
+            basis.c =
+                NSCoordSaturatingSubtract(basis.c, col_iSize, nscoord_MAX);
+            col_iSize = NSCoordSaturatingAdd(
+                col_iSize, NSToCoordRound(float(col_iSize) * c));
           }
         }
         break;
@@ -935,13 +970,13 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
                      "wrong case");
         if (pct != 0.0f) {
           float c = float(space) / basis.f;
-          col_iSize += NSToCoordRound(pct * c);
+          col_iSize = NSCoordSaturatingAdd(col_iSize, NSToCoordRound(pct * c));
           basis.f -= pct;
         }
         break;
       case FLEX_ALL_LARGE: {
         float c = float(space) / float(basis.c);
-        col_iSize += NSToCoordRound(c);
+        col_iSize = NSCoordSaturatingAdd(col_iSize, NSToCoordRound(c));
         --basis.c;
       } break;
     }
@@ -961,20 +996,20 @@ void BasicTableLayoutStrategy::DistributeISizeToColumns(
 
     // Apply the new isize
     switch (aISizeType) {
-      case BTLS_MIN_ISIZE: {
+      case BtlsISizeType::MinISize: {
         // Note: AddSpanCoords requires both a min and pref isize.
         // For the pref isize, we'll just pass in our computed
         // min isize, because the real pref isize will be at least
         // as big
         colFrame->AddSpanCoords(col_iSize, col_iSize, aSpanHasSpecifiedISize);
       } break;
-      case BTLS_PREF_ISIZE: {
+      case BtlsISizeType::PrefISize: {
         // Note: AddSpanCoords requires both a min and pref isize.
         // For the min isize, we'll just pass in 0, because
         // the real min isize will be at least 0
         colFrame->AddSpanCoords(0, col_iSize, aSpanHasSpecifiedISize);
       } break;
-      case BTLS_FINAL_ISIZE: {
+      case BtlsISizeType::FinalISize: {
         nscoord old_final = colFrame->GetFinalISize();
         colFrame->SetFinalISize(col_iSize);
 
